
/*****************************************************************************
 *
 * File: Phoenix_Adapter.ddl
 * Version: 4.1.0
 * Date: 07/06/10
 *
 *****************************************************************************
 * Copyright (c) 2010, HART Communication Foundation, All Rights Reserved
 *****************************************************************************
 *
 * Description: This file is a sample DD intended to demonstrate the 
 *              development of a simple wireless adapter DD.  It supports HART 7, 
 *              Universal Commands, a few Common Practice Commands, and all the 
 *              WirelessHART commands.
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
 */
// Replace following line with your manufacturer and device type as defined in Devices.cfg
// This is a HART 7 device so we will need treat the device type differently for legacy
// hosts (ie, .fms and .fm6)
//fbi	 #if __TOKVER__ >= 800
//fbi	 MANUFACTURER Phoenix_Contact, DEVICE_TYPE _RAD_WHA, DEVICE_REVISION 1, DD_REVISION 1
//fbi	 #else

//fbi	 MANUFACTURER HCF, DEVICE_TYPE _SAMPLE3_ADAPTER, DEVICE_REVISION 1, DD_REVISION 1
//fbi	 #endif

MANUFACTURER 0x00B0, DEVICE_TYPE 0xB011, DEVICE_REVISION 1, DD_REVISION 1




/*****************************************************************************
 * Include Files
 *****************************************************************************
 */

#include "macros.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */

/* 
 * Because the number of bytes in the response of event commands
 * is not pre-determined, the event commands should not be imported
 * at this time. This is because the adapter does not know how many command 48
 * bytes that are implemented in the subdevice, and 1 adapter can be mapped to
 * multiple subdevices each with different lengths of command 48.
 * Set to 1 to include all event data bytes - but warning is that it is
 * possible to get not enough response bytes back from the command.
 */
 
#define INCLUDE_EVENTS 0 
#define __TOKVER__ 800
/*****************************************************************************
 * Common Tables
 */

//fbi	IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
IMPORT "Tables_20_4.ddl"
{
    // Import all so that we get all device types for subdevices attached to the adapter.
    EVERYTHING;

    REDEFINITIONS
    {
        VARIABLE device_type
        {
            /* 
             * The type must be redefined if the Device Type is not listed
             * in Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0xf9f7 with your Expanded Device Type code and appropriate label                 
                { 0xB011,  "_RAD_WHA" }  
            }
        }  

//jad
//add to list, not just replace
        VARIABLE subdev_expanded_device_type
        {
            TYPE ENUMERATED
            {
                ADD { 0xB011,  "_RAD_WHA" }             //Not sure this is the correct text
            }
        }

        VARIABLE manufacturer_id
        {
            /*
             * The type must be redefined if the Manufacturer code is not listed
             * in the Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your Manufacturer code and appropriate label
                //{ 0x00f9,  [HART_Communication_Foundation] } 
                   { 0x00B0,  [Phoenix_Contact] } 
               
            }
        }

        VARIABLE private_label_distributor
        {
            /*
             * The type must be redefined if the Private Label code is not listed
             * in the Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)  
            {
                // Replace the 0x00f9 with your manufactures id and appropriate label
                //{ 0x00f9,  [HART_Communication_Foundation] } 
                  { 0x00B0,  [Phoenix_Contact] }
            }
        }

        VARIABLE burst_mode_select
        {
            REDEFINE DEFAULT_VALUE 2;
            REDEFINE CLASS DYNAMIC;
            REDEFINE TYPE ENUMERATED
            {
                BURST_MODE_CONTROL_CODE(0),  /* Off */
                { 2,  "On",  "Enabled Burst Mode" }
            }
        }


        /* 
         * Remove the following if the device supports trends.  Corresponding
         * commands will also need to be imported from the COMMON_PRACTICE DD.
         */
        DELETE VARIABLE trend_digital_units;
        DELETE VARIABLE trend_classification;
        DELETE VARIABLE trend_control_codes;
        DELETE VARIABLE trend_control;

        // Commands 821,822 are optional for wireless devices - remove if device does not support
        DELETE VARIABLE network_access_mode;

        // Command 64,512 is optional for wireless devices - remove if device does not support
        DELETE VARIABLE wireless_module_device_type;
        DELETE VARIABLE wireless_module_manufacturer_id;

        // Command 89 is not typically supported by wireless adapters (they get time from the network manager)
        DELETE VARIABLE time_set;

        // Command 7 loop current mode not supported , instead new command 128 is supported for the same but it is used in command 6
//jad
// needed for read_loop_configuration
//        DELETE VARIABLE loop_current_mode;

#if __TOKVER__ < 800
        DELETE VARIABLE real_time_clock_flag;
#endif
/*fbi
#if (__TOKVER__ < 600) || !(INCLUDE_EVENTS)
        DELETE VARIABLE event_nofication_control_codes;
        DELETE VARIABLE event_status_codes;
        DELETE VARIABLE event_status;
        DELETE VARIABLE event_notification_control_n;
        DELETE VARIABLE device_status_mask;
        DELETE VARIABLE extended_fld_device_status_mask;
        DELETE VARIABLE standardized_status_0_mask;
        DELETE VARIABLE standardized_status_1_mask;
        DELETE VARIABLE standardized_status_2_mask;
        DELETE VARIABLE standardized_status_3_mask;
        DELETE VARIABLE analog_channel_saturated1_mask;
        DELETE VARIABLE analog_channel_fixed1_mask;
        DELETE VARIABLE device_status_latched_value;
        DELETE VARIABLE dev_operating_mode_mask;
        DELETE VARIABLE extended_fld_device_status_latched_value;
        DELETE VARIABLE standardized_status_0_latched_value;
        DELETE VARIABLE standardized_status_1_latched_value;
        DELETE VARIABLE standardized_status_2_latched_value;
        DELETE VARIABLE standardized_status_3_latched_value;
        DELETE VARIABLE analog_channel_saturated1_latched_value;
        DELETE VARIABLE analog_channel_fixed1_latched_value;
        DELETE VARIABLE dev_operating_mode_latched_value;
#endif
*/
    }
}
/*****************************************************************************
 * Universal Commmands
 */

//fbi	IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
IMPORT "Universal_7_2.ddl"
{
    // All HART devices must support all Universal Commands
    EVERYTHING;

    REDEFINITIONS
    {
         
         /* The transaction will need to be redefined to show the correct
         * number of dynamic variables returned in this command 
         * (Command 3) 
         */
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE
                }
            }
        }

        /* 
         * The transaction will need to be redefined to show the correct
         * number of dynamic variables returned in this command 
         * (Command 8)
         */
        COMMAND read_dynamic_variable_classification
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DEVICE_VARIABLE.CLASSIFICATION,
                    250,
                    250,
                    250
                }
            }
        }
//jad
//may be able to use this version...
/*
        COMMAND write_polling_address
        {

            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    polling_address
                }
                REPLY
                {
                    response_code, device_status, polling_address
                }
            }

        }
*/
 
        // Remove the command 48 status bytes that are not supported by the device
        // See also command 48 related items in the common practice imports.
        // 
       
        DELETE VARIABLE device_specific_status_14;
        DELETE VARIABLE device_specific_status_15;
        DELETE VARIABLE device_specific_status_16;
        DELETE VARIABLE device_specific_status_17;
        DELETE VARIABLE device_specific_status_18;
        DELETE VARIABLE device_specific_status_19;
        DELETE VARIABLE device_specific_status_20;
        DELETE VARIABLE device_specific_status_21;
        DELETE VARIABLE device_specific_status_22;
        DELETE VARIABLE device_specific_status_23;
        DELETE VARIABLE device_specific_status_24;
//jad
// Needed to read polling address
// May be able to redefine Command 7 to remove var below
//        DELETE COMMAND read_loop_configuration;
        DELETE COMMAND read_device_variables_and_status;
       // DELETE COMMAND read_pv_sensor_info;
       // DELETE COMMAND read_pv_output_info;


        /* 
         * Redefine the command 48 status to match
         * the device implementation.  The same should
         * also be done for device_specific_status_0_mask
         * and device_specific_status_0_latched_value
         * example for 1 byte follows: 
         */
        VARIABLE device_specific_status_0
        {
            REDEFINE LABEL "Status example 1";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Example condition 1", MISC },
                { 0x02, "Example condition 2", MISC },
                { 0x04, "Example condition 3", MISC },
                { 0x08, "Example condition 4", MISC },
                { 0x10, "Example condition 5", MISC },
                { 0x20, "Example condition 6", MISC },
                { 0x40, "Example condition 7", MISC },
                { 0x80, "Example condition 8", MISC }
            }
        }

//fbi
	 VARIABLE max_num_device_variables
         {
             REDEFINE LABEL "Max DevVars";
         }
         VARIABLE device_id
         {
             REDEFINE LABEL "Dev ID";
         }
         VARIABLE hardware_revision
         {
             REDEFINE LABEL "Hardware Rev";
         }
         VARIABLE software_revision
         {
             REDEFINE LABEL "Software Rev";
         }

        VARIABLE final_assembly_number
        {
            REDEFINE LABEL "Final assembly number";
 
        }
        VARIABLE polling_address
        {
            REDEFINE LABEL "Polling Address";     
        }
	VARIABLE tag
	{
	 REDEFINE CLASS DYNAMIC;
	}
	
//jad
//moved from common practice
        VARIABLE config_change_counter
         {
            REDEFINE LABEL "CFG Change Count";
         }

        // Remove the command 48 status bytes that are not supported by the adapter
        // See also command 48 related items in the common practice imports.
        COMMAND read_additional_device_status
        {
            REDEFINE TRANSACTION 
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,

                    extended_fld_device_status, 0x00,

                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }

            REDEFINE TRANSACTION 1
            {
                REQUEST
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO)
                }
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO)
                }
            }
        }
    }
}

/*****************************************************************************
 * PV, when the primary variable is not mappable we can import the PV
 * standard DD to take care of the dynamic variable array, loop current
 * and ranging functions
 */

//fbi	IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
IMPORT "pv1_1.ddl"
{
    EVERYTHING;


    REDEFINITIONS
    {
        /*
         * Remove the device variables not used in this DD.
         */
        DELETE COLLECTION secondary_variable;
        DELETE COLLECTION tertiary_variable;
        DELETE COLLECTION quaternary_variable;

        ARRAY OF COLLECTION dynamic_variables
        {
            ELEMENTS
            {
                DELETE SECONDARY;
                DELETE TERTIARY;
                DELETE QUATERNARY;
            }
        }

        COLLECTION OF COLLECTION primary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE,        mappableVariables[0];
            }
        }

        //COLLECTION OF COLLECTION secondary_variable
        //{
        //    MEMBERS
        //    {
        //        REDEFINE DEVICE_VARIABLE,        mappableVariables[1];
        //    }
        //}

        //COLLECTION OF COLLECTION tertiary_variable
        //{
        //    MEMBERS
        //    {
        //        REDEFINE DEVICE_VARIABLE,        mappableVariables[2];
        //    }
        //}

        //COLLECTION OF COLLECTION quaternary_variable
        //{
        //    MEMBERS
        //    {
        //        REDEFINE DEVICE_VARIABLE,        mappableVariables[3];
        //    }
        //}

        /* 
         * Most devices use the same data item for range units (command 15) and the sensor units.
         * when this is true the following should be added.  if your device uses a different variable
         * for range units you should reference it here 
         */
        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD RANGE_UNITS, mappableVariables[0].DIGITAL_UNITS, [range_units], [range_units_help];
            } 
        }

        REDEFINE UNIT scaling_units_relation
        {
            mappableVariables[0].DIGITAL_UNITS:
            upperRange_value,
            lowerRange_value
        }
    }
}


/*****************************************************************************
 * Common Practice Commands
 */

//fbi	IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
IMPORT "Common_9.ddl"
{
    // For a wireless adapter, these are the minimal commands required (in the device) for compliance.
    // Commented items are required in the device, but not modeled in a DD

    
    COMMAND read_io_system_capabilities;                 // Command 74
    COMMAND read_subdev_id_summary;                     // Command 84
    COMMAND read_subdevice_burst_map;                   // Command 101
    COMMAND write_subdevice_burst_map;                  // Command 102
    COMMAND write_burst_period;                         // Command 103
    COMMAND write_burst_trigger;                        // Command 104
    COMMAND read_burst_mode_configuration;              // Command 105
    //COMMAND flush_delayed_responses;                    // Command 106
    COMMAND write_burst_device_variables;               // Command 107
    COMMAND write_burst_command_number;                 // Command 108
    COMMAND burst_mode_control;                         // Command 109
                          
    

    VARIABLE max_io_cards;                              // for Command 74
    VARIABLE max_channels_per_io_card;                  // for Command 74
    VARIABLE max_sub_dev_per_channel;                   // for Command 74
    VARIABLE devices_detected;                          // for Command 74
    VARIABLE max_dr;                                    // for Command 74
    VARIABLE retry_count;                               // for Command 74, 88
//jad
//should be in univeral import area
//    VARIABLE config_change_counter;     
    
    
#if __TOKVER__ >= 800
//    VARIABLE subdevice_number;                          // for Command 84, 86
    LIST     subdevice_list;                            // for Command 84, 86
    VARIABLE IO_card_number;                            // for Command 85
// mhu   VARIABLE IO_channel_number;                         // for Command 85
    ARRAY card0_channels;                               // for ARRAY io_card_list
    ARRAY OF ARRAY io_cards;                            // for Command 85

    VARIABLE burst_message_number;                      // for Command 103, 104, 105, 107, 108, 109
    VARIABLE update_period;                             // for collection burst_message_collection
    VARIABLE max_update_period;                         // for collection burst_message_collection
    VARIABLE burst_variable_code;                       // for collection burst_message_collection
    ARRAY    burst_variable_codes;                      // for collection burst_message_collection
    VARIABLE burst_subdevice_index;                     // for Command 101, 102
// mhu    ARRAY burst_messages;                               // for Command 103, 104, 105, 107, 108, 109

#if INCLUDE_EVENTS
    VARIABLE eventNumber;                               // for Command 115, 116, 117, 118, 119
    VARIABLE time_of_first_unack_event;                 // for collection event_control
    VARIABLE event_notification_retry_time_event;       // for collection event_control
    VARIABLE max_update_time_event;                     // for collection event_control
    VARIABLE event_debounce_interval_event;             // for collection event_control
    VARIABLE event_subdevice_index;                     // for Command 101, 102
    ARRAY events;                                       // for Command 115, 116, 117, 118, 119
#endif

#else
    VARIABLE subdevice_number_int;                      // for Command 84, 86
    VARIABLE IO_card_number_int;                        // for Command 85
    VARIABLE IO_channel_number_int;                     // for Command 85
    VARIABLE update_period_int;                         // for collection burst_message_collection
    VARIABLE max_update_period_int;                     // for collection burst_message_collection
//    VARIABLE burst_variable_code_1;                     // for Command 105, 107
//    VARIABLE burst_variable_code_2;                     // for Command 105, 107                                                        // 
//    VARIABLE burst_variable_code_3;                     // for Command 105, 107
//    VARIABLE burst_variable_code_4;                     // for Command 105, 107
//    VARIABLE burst_variable_code_5;                     // for Command 105, 107
//    VARIABLE burst_variable_code_6;                     // for Command 105, 107
//    VARIABLE burst_variable_code_7;                     // for Command 105, 107
 //   VARIABLE burst_variable_code_8;                     // for Command 105, 107
    VARIABLE burst_subdevice_index_int;                 // for Command 101, 102

#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
    VARIABLE time_of_first_unack_event_int;             // for collection event_control
    VARIABLE event_notification_retry_time_event_int;   // for collection event_control
    VARIABLE max_update_time_event_int;                 // for collection event_control
    VARIABLE event_debounce_interval_event_int;         // for collection event_control
    VARIABLE event_subdevice_index_int;                 // for Command 101, 102
#endif
#endif

    VARIABLE STX_count;                                 // for Command 95
    VARIABLE ACK_count;                                 // for Command 95
    VARIABLE BACK_count;                                // for Command 95
    VARIABLE total_number_burst_messages;               // for Command 105
    VARIABLE subdev_IO_card;                            // for collection subdev_identity
    VARIABLE subdev_channel;                            // for collection subdev_identity
    VARIABLE subdev_device_id;                          // for collection subdev_identity
    VARIABLE subdev_universal_command_revision;         // for collection subdev_identity
    VARIABLE subdev_long_tag;                           // for collection subdev_identity
    VARIABLE stx_sent_to_subdev;                        // for collection subdev_stats
    VARIABLE ack_received_from_subdev;                  // for collection subdev_stats
    VARIABLE back_received_from_subdev;                 // for collection subdev_stats
    VARIABLE io_channel_stx_sent;                       // for collection io_stats
    VARIABLE io_channel_ack_received;                   // for collection io_stats
    VARIABLE io_channel_ostx_received;                  // for collection io_stats
    VARIABLE io_channel_oack_received;                  // for collection io_stats
    VARIABLE io_channel_back_received;                  // for collection io_stats
    VARIABLE burst_command_number;                      // for collection burst_message_collection
    VARIABLE trigger_level;                             // for collection burst_message_collection

#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
    VARIABLE number_events_supported;                   // for Command 115
    VARIABLE device_specific_status_0_mask;             // for collection event_mask
    VARIABLE device_specific_status_1_mask;             // for collection event_mask
    VARIABLE device_specific_status_2_mask;             // for collection event_mask
    VARIABLE device_specific_status_3_mask;             // for collection event_mask
    VARIABLE device_specific_status_4_mask;             // for collection event_mask
    VARIABLE device_specific_status_5_mask;             // for collection event_mask

    /*
     * Adapters handle events on behalf of subdevices,
     * so ALL status masks must be imported so that
     * subdevices with any number of status bytes can
     * be supported.
     */
    VARIABLE device_specific_status_14_mask;            // for collection event_mask
    VARIABLE device_specific_status_15_mask;            // for collection event_mask
    VARIABLE device_specific_status_16_mask;            // for collection event_mask
    VARIABLE device_specific_status_17_mask;            // for collection event_mask
    VARIABLE device_specific_status_18_mask;            // for collection event_mask
    VARIABLE device_specific_status_19_mask;            // for collection event_mask
    VARIABLE device_specific_status_20_mask;            // for collection event_mask
    VARIABLE device_specific_status_21_mask;            // for collection event_mask
    VARIABLE device_specific_status_22_mask;            // for collection event_mask
    VARIABLE device_specific_status_23_mask;            // for collection event_mask
    VARIABLE device_specific_status_24_mask;            // for collection event_mask
    VARIABLE device_specific_status_0_latched_value;    // for collection event_report
    VARIABLE device_specific_status_1_latched_value;    // for collection event_report
    VARIABLE device_specific_status_2_latched_value;    // for collection event_report
    VARIABLE device_specific_status_3_latched_value;    // for collection event_report
    VARIABLE device_specific_status_4_latched_value;    // for collection event_report
    VARIABLE device_specific_status_5_latched_value;    // for collection event_report

    /*
     * Adapters handle events on behalf of subdevices,
     * so ALL status masks must be imported so that
     * subdevices with any number of status bytes can
     * be supported.
     */
    VARIABLE device_specific_status_14_latched_value;   // for collection event_report
    VARIABLE device_specific_status_15_latched_value;   // for collection event_report
    VARIABLE device_specific_status_16_latched_value;   // for collection event_report
    VARIABLE device_specific_status_17_latched_value;   // for collection event_report
    VARIABLE device_specific_status_18_latched_value;   // for collection event_report
    VARIABLE device_specific_status_19_latched_value;   // for collection event_report
    VARIABLE device_specific_status_20_latched_value;   // for collection event_report
    VARIABLE device_specific_status_21_latched_value;   // for collection event_report
    VARIABLE device_specific_status_22_latched_value;   // for collection event_report
    VARIABLE device_specific_status_23_latched_value;   // for collection event_report
    VARIABLE device_specific_status_24_latched_value;   // for collection event_report
    VARIABLE config_change_counter_latched_value;       // for collection event_report
    COLLECTION event_control;                           // for ARRAY event_array
    COLLECTION event_mask;                              // for ARRAY event_array
    COLLECTION event_report;                            // for ARRAY event_array
    COLLECTION event;                                   // for ARRAY event_array
#endif

    VARIABLE constOne;                                  // for command 101,102
    VARIABLE constZero;                                 // for command 101,102
    VARIABLE subdevice_missing;                         // for command 101,102
   
    COLLECTION subdev_identity;                         // for collection sub_device
    COLLECTION subdev_stats;                            // for collection sub_device
    COLLECTION sub_device;                              // for LIST subdevice_list
    COLLECTION io_stats;                                // for LIST channel_list
    COLLECTION burst_message_collection;                // for ARRAY burst_message_array

    METHOD return_to_normal;
    METHOD warning_message;

    REDEFINITIONS
    {
// mhu added to set index to new defined burst_messages
		 VARIABLE burst_message_number
		 {
			REDEFINE TYPE INDEX burst_messages2;
		 }
// mhu finish		 
		// Define how many burst messages this adapter supports
#if __TOKVER__ >= 800
        ARRAY burst_messages
        {
            // A minimum of 4 are required
            REDEFINE NUMBER_OF_ELEMENTS 4;
        }

        // Define how many events this adapter supports
#if INCLUDE_EVENTS
        ARRAY events
        {
            // A minimum of 2 are required
            REDEFINE NUMBER_OF_ELEMENTS 2;
        }
#endif
// mhu EDDY doesn't take ARRAY for INDEX VARIABLES (use ITEM_ARRAY instead) ... anyway not needed here
#ifndef _EDDY_
        ARRAY card0_channels
        {
            // How many IO channels exist on IO card
            REDEFINE NUMBER_OF_ELEMENTS 1;
        }
#endif

#endif
         VARIABLE max_sub_dev_per_channel
         {
             REDEFINE LABEL "Max Sub-devices per channel";
         }

         VARIABLE max_channels_per_io_card
         {
             REDEFINE LABEL "Max Channels Per I/O Card";
         }
         VARIABLE max_io_cards
         {
             REDEFINE LABEL "Max Number I/O cards";
         }
         VARIABLE total_number_burst_messages
         {
            REDEFINE LABEL "Total # Burst Messages";
         }
//jad
//moved to universal import
/*
         VARIABLE config_change_counter
         {
            REDEFINE LABEL "CFG Change Count";
         }
*/
/*
         VARIABLE max_num_device_variables
         {
             REDEFINE LABEL "Max DevVars";
         }
         VARIABLE device_id
         {
             REDEFINE LABEL "Dev ID";
         }
         VARIABLE hardware_revision
         {
             REDEFINE LABEL "Hardware Rev";
         }
         VARIABLE software_revision
         {
             REDEFINE LABEL "Software Rev";
         }
*/
//        VARIABLE burst_subdevice_index
//        {
 //           REDEFINE TYPE INDEX (2) subdevice_array;
//        }

//jad
//needed because AMS does not support LIST constructs
COMMAND read_subdev_id_summary      
{
    REDEFINE TRANSACTION
    {

        REQUEST
        {
            subdevice_number_array_x (INFO),        //jad, bug in ams with 2 byte indexes in arrays
            subdevice_number_array (INFO, INDEX)
        }
        REPLY
        {
            response_code, device_status,
            subdevice_number_array_x (INFO),        //jad, bug in ams with 2 byte indexes in arrays
            subdevice_number_array (INFO, INDEX), 
            subdevice_array[subdevice_number_array].IO_CARD,
//            subdevice_list[subdevice_number_array].IDENTITY.IO_CARD,
            subdevice_array[subdevice_number_array].CHANNEL,                  
            subdevice_array[subdevice_number_array].MANUFID,                 
            subdevice_array[subdevice_number_array].EXP_DEV_TYPE, 
            subdevice_array[subdevice_number_array].DEV_ID,
            subdevice_array[subdevice_number_array].UNIV_CMD_REV,  
            subdevice_array[subdevice_number_array].LONG_TAG

//            subdevice_list[subdevice_number_array].IDENTITY.IO_CARD
        }
    }

}

// mhu added - COMMAND write_burst_trigger must use newly defined burst_messages2
COMMAND write_burst_trigger
{
	REDEFINE TRANSACTION
	{
#if __TOKVER__ >= 800
        REQUEST
        {
            burst_message_number (INFO, INDEX),
            burst_messages2[burst_message_number].BURST_TRIGGER_MODE,
            burst_messages2[burst_message_number].BURST_CLASSIFICATION,
            burst_messages2[burst_message_number].BURST_UNIT,
            burst_messages2[burst_message_number].TRIGGER_LEVEL
        }
        REPLY
        {
            response_code, device_status,
            burst_message_number (INFO, INDEX),
            burst_messages2[burst_message_number].BURST_TRIGGER_MODE,
            burst_messages2[burst_message_number].BURST_CLASSIFICATION,
            burst_messages2[burst_message_number].BURST_UNIT,
            burst_messages2[burst_message_number].TRIGGER_LEVEL
        }
#else
        REQUEST
        {
            0,
            burst_message_collection.BURST_TRIGGER_MODE,
            burst_message_collection.BURST_CLASSIFICATION,
            burst_message_collection.BURST_UNIT,
            burst_message_collection.TRIGGER_LEVEL
        }
        REPLY
        {
            response_code, device_status,
            0,
            burst_message_collection.BURST_TRIGGER_MODE,
            burst_message_collection.BURST_CLASSIFICATION,
            burst_message_collection.BURST_UNIT,
            burst_message_collection.TRIGGER_LEVEL
        }
#endif
     }
  }
// mhu finish
        // Define the list of commands that can be burst by the adapter (add any device specific commands too)
       /* VARIABLE burst_command_number
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                {1,  [read_primary_variable], [pv_help] }, 
                {2,  [read_percent_range_and_current], [percent_rnge_help] }, 
                {3,  [read_dyn_vars_and_current], [all_process_var_help] }, 
                {9,  [read_device_variables_with_status], [read_device_variables_with_status_help] }, 
                {33, [read_device_variables],[read_device_variables_help] },
                {48, "Cmd 48: Read Additional Device Status", "Read Additional Device - This command reads additional device status information"}               
            }
        }*/

//fbi	
/*
#if __TOKVER__ >= 800
    VARIABLE update_period 
    {
        REDEFINE TYPE TIME_VALUE
        { 
            
            DISPLAY_FORMAT "5.3f" ;
            TIME_SCALE SECONDS ;
            
        }
        REDEFINE DEFAULT_VALUE 60;        
        
    }

*/
//fbi	#else

VARIABLE update_period_int 
{
	REDEFINE LABEL "Update Period";
	REDEFINE HELP "Update period for burst message";
	REDEFINE CLASS DEVICE;
	REDEFINE TYPE UNSIGNED_INTEGER(4)
	{
		SCALING_FACTOR 0.00003125;
	}
	REDEFINE CONSTANT_UNIT "s";
    }


//fbi	#endif


     VARIABLE burst_command_number
      {
          REDEFINE LABEL "Burst process command";
          REDEFINE DEFAULT_VALUE 1;
          REDEFINE TYPE ENUMERATED (2)
          {
              { 0, "Process Data Burst Message Off", "help" },
              { 1, "Command 1", "help" },
              { 2, "Command 2", "help" },
              { 3, "Command 3", "help" },
              { 9, "Command 9", "help" } //Commented on 27/01/2012
           }
       }
       /*fbi
     VARIABLE tag
     {
         REDEFINE CLASS DYNAMIC;
     }
       */

 /*   #if __TOKVER__ >= 600
    COLLECTION burst_message_collection
    #else
    COLLECTION OF VARIABLE burst_message_collection
    #endif
    {
        MEMBERS {

            REDEFINE COMMAND_NUMBER,         ProcessCommand;


        }
    }*/



#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
        /* The event masks and reported values should be
           redefined similar to the command 48 byte
           redefinitions in the UNIVERSAL imports. */
        VARIABLE device_specific_status_0_mask
        {
            REDEFINE LABEL "Status example 1 Mask";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Example condition 1", MISC },
                { 0x02, "Example condition 2", MISC },
                { 0x04, "Example condition 3", MISC },
                { 0x08, "Example condition 4", MISC },
                { 0x10, "Example condition 5", MISC },
                { 0x20, "Example condition 6", MISC },
                { 0x40, "Example condition 7", MISC },
                { 0x80, "Example condition 8", MISC }
            }
        }

        VARIABLE device_specific_status_0_latched_value
        {
            REDEFINE LABEL "Latched Status example 1";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Example condition 1", MISC },
                { 0x02, "Example condition 2", MISC },
                { 0x04, "Example condition 3", MISC },
                { 0x08, "Example condition 4", MISC },
                { 0x10, "Example condition 5", MISC },
                { 0x20, "Example condition 6", MISC },
                { 0x40, "Example condition 7", MISC },
                { 0x80, "Example condition 8", MISC }
            }
        }
#else
        /* If events are not included, then the command 101 transaction
           dealing with the event notifications needs to be removed. */
        REDEFINE COMMAND read_subdevice_burst_map
        {
            NUMBER 101;
            OPERATION READ;
#if __TOKVER__ >= 800
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    burst_message_number <0x7F> (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    burst_message_number <0x7F> (INFO, INDEX),
//JAD
//                    burst_messages[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
//                    burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                    burst_messages2[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
                    burst_messages2[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
#else
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    constZero <0x7F> (INFO)
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    constZero <0x7F> (INFO),
                    burst_message_collection.SUB_DEVICE_MAPPING
                }
            }
#endif
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
            }
        }
         

        /* If events are not included, then the command 102 transaction
           dealing with the event notifications needs to be removed. */
        REDEFINE COMMAND write_subdevice_burst_map
        {
            NUMBER 102;
            OPERATION WRITE;
#if __TOKVER__ >= 800
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    burst_message_number <0x7F> (INFO, INDEX),
// mhu - data type incorrect 		    
//                    constZero <0x8000>,
                    constZero2 <0x8000>,
//JAD
//                    burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
// mhu - only copy & paste error fixed
                    burst_messages2[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    burst_message_number <0x7F> (INFO, INDEX),
//JAD
//                    burst_messages[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
//                    burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                    burst_messages2[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
                    burst_messages2[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
#else
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    constZero <0x7F> (INFO),
                    burst_message_collection.SUB_DEVICE_MAPPING
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    constZero <0x7F> (INFO),
                    burst_message_collection.SUB_DEVICE_MAPPING
                }
            }
#endif
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                9,  DATA_ENTRY_ERROR, "Invalid subdevice index";
                16, MODE_ERROR,       [access_restricted];
            }
        }
#endif

//jad 
    COMMAND read_burst_mode_configuration
    {
        REDEFINE TRANSACTION
        {
            REQUEST 
            {
                burst_message_number (INFO, INDEX)
            }
            REPLY
            {
                response_code, device_status,
                burst_messages2[burst_message_number].BURST_MODE_SELECT,
                0x1F,
                burst_messages2[burst_message_number].BURST_VAR_1,        
                burst_messages2[burst_message_number].BURST_VAR_2,        
                burst_messages2[burst_message_number].BURST_VAR_3,        
                burst_messages2[burst_message_number].BURST_VAR_4,        
                burst_messages2[burst_message_number].BURST_VAR_5,        
                burst_messages2[burst_message_number].BURST_VAR_6,        
                burst_messages2[burst_message_number].BURST_VAR_7,        
                burst_messages2[burst_message_number].BURST_VAR_8, 
                burst_message_number (INFO, INDEX),
                total_number_burst_messages,
                burst_messages2[burst_message_number].COMMAND_NUMBER,
                burst_messages2[burst_message_number].UPDATE_PERIOD,
                burst_messages2[burst_message_number].MAX_UPDATE_PERIOD,
                burst_messages2[burst_message_number].BURST_TRIGGER_MODE,
                burst_messages2[burst_message_number].BURST_CLASSIFICATION,
                burst_messages2[burst_message_number].BURST_UNIT,
                burst_messages2[burst_message_number].TRIGGER_LEVEL
            }
        }
    }

    COMMAND write_burst_period
    {
    REDEFINE TRANSACTION
    {

        REQUEST
        {
            burst_message_number (INFO, INDEX),
            burst_messages2[burst_message_number].UPDATE_PERIOD,
            burst_messages2[burst_message_number].MAX_UPDATE_PERIOD
        }
        REPLY
        {
            response_code, device_status,
            burst_message_number (INFO, INDEX),
            burst_messages2[burst_message_number].UPDATE_PERIOD,
            burst_messages2[burst_message_number].MAX_UPDATE_PERIOD
        }
    }
    }
    COMMAND write_burst_device_variables
    {
    REDEFINE TRANSACTION
    {
        REQUEST
        {
                burst_messages2[burst_message_number].BURST_VAR_1,        
                burst_messages2[burst_message_number].BURST_VAR_2,        
                burst_messages2[burst_message_number].BURST_VAR_3,        
                burst_messages2[burst_message_number].BURST_VAR_4,        
                burst_messages2[burst_message_number].BURST_VAR_5,        
                burst_messages2[burst_message_number].BURST_VAR_6,        
                burst_messages2[burst_message_number].BURST_VAR_7,        
                burst_messages2[burst_message_number].BURST_VAR_8,
               burst_message_number (INFO, INDEX)
        }
        REPLY
        {
            response_code, device_status,
                burst_messages2[burst_message_number].BURST_VAR_1,        
                burst_messages2[burst_message_number].BURST_VAR_2,        
                burst_messages2[burst_message_number].BURST_VAR_3,        
                burst_messages2[burst_message_number].BURST_VAR_4,        
                burst_messages2[burst_message_number].BURST_VAR_5,        
                burst_messages2[burst_message_number].BURST_VAR_6,        
                burst_messages2[burst_message_number].BURST_VAR_7,        
                burst_messages2[burst_message_number].BURST_VAR_8,
               burst_message_number (INFO, INDEX)
        }
    }
}
    COMMAND write_burst_command_number
    {
    REDEFINE TRANSACTION
    {
        REQUEST
        {
            burst_messages2[burst_message_number].COMMAND_NUMBER,
            burst_message_number (INFO, INDEX)
        }
        REPLY
        {
            response_code, device_status,
            burst_messages2[burst_message_number].COMMAND_NUMBER,
            burst_message_number (INFO, INDEX)
        }

    }

}
COMMAND burst_mode_control
{
    REDEFINE TRANSACTION
    {
        REQUEST
        {
            burst_messages2[burst_message_number].BURST_MODE_SELECT,
            burst_message_number (INFO, INDEX)
        }
        REPLY
        {
            response_code, device_status,
            burst_messages2[burst_message_number].BURST_MODE_SELECT,
            burst_message_number (INFO, INDEX)
        }

    }
}
}

}


//fbi	IMPORT STANDARD _WIRELESS, DEVICE_REVISION 1, DD_REVISION 2
IMPORT "Wireless.ddl"
{
    COMMAND write_join_key;      //cmd 768
    COMMAND read_join_status;  //cmd 769
    COMMAND write_network_id;  //cmd 773
    COMMAND read_network_id;     //cmd 774
 
    VARIABLE network_id;
    VARIABLE device_nickname_address;
    VARIABLE join_key_1;
    VARIABLE join_key_2;
    VARIABLE join_key_3;
    VARIABLE join_key_4;

    REDEFINITIONS
    {
        // Commands 821,822 are optional for adapters - remove if device does not support
        /*DELETE COMMAND write_network_access_mode;
        DELETE COMMAND read_network_access_mode;

        // Commands 64,512 are optional for adapters - remove if device does not support
        DELETE VARIABLE wireless_module_transmitter_revision;
        DELETE VARIABLE wireless_module_software_revision;
        DELETE VARIABLE wireless_module_hardware_revision;
        DELETE COMMAND read_wireless_module_revision;*/
        //DELETE VARIABLE neighbor_count;
        //DELETE VARIABLE advertisements;
        //DELETE VARIABLE join_attempts;
        //DELETE VARIABLE join_retry_timer;
        //DELETE VARIABLE network_search_timer;
    
    
    
        REDEFINE COMMAND read_join_status
        {
            NUMBER 769;
            OPERATION READ;
//fbi	 #if __TOKVER__ >= 800
            TRANSACTION
            {
                REQUEST
                {
                }
                REPLY
                {
//jad extra bytes seen
//put back to normal, maybe version 144 fixed
//                    response_code, device_status, Join_Status
//NOPE!
                     response_code, device_status, Join_Status, jad_junk_1
                }
            }
//fbi	 #endif
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
                4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                16, MODE_ERROR,       [access_restricted];
                32, MODE_ERROR,       [busy];
            }
        }
   /*REDEFINE COMMAND read_network_id
        {
            NUMBER 774;
            OPERATION READ;
 #if __TOKVER__ >= 800
            TRANSACTION
            {
                REQUEST
                {
                }
                REPLY
                {
                    response_code, device_status, network_id
                }
            }
#endif
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
                4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                16, MODE_ERROR,       [access_restricted];
                32, MODE_ERROR,       [busy];
            }
    }*/

 
        

        VARIABLE join_key_1
        {             
             REDEFINE LABEL "Join Key 1";             
             REDEFINE DEFAULT_VALUE 0xFFFFFFFF;
        }
        VARIABLE join_key_2
        {
            REDEFINE LABEL "Join Key 2";
            REDEFINE DEFAULT_VALUE 0xFFFFFFFF;
        }
        VARIABLE join_key_3
        {
            REDEFINE LABEL "Join Key 3";
            REDEFINE DEFAULT_VALUE 0xFFFFFFFF;
        }
        VARIABLE join_key_4
        {
           REDEFINE LABEL "Join Key 4";
           REDEFINE DEFAULT_VALUE 0xFFFFFFFF;
        }

        VARIABLE network_id
        {
            REDEFINE DEFAULT_VALUE 0x01;
            REDEFINE CLASS DYNAMIC;
            REDEFINE TYPE UNSIGNED_INTEGER (2)
            {
                MAX_VALUE  0x8FFF;   // reference Spec 75, TDMA Data Link Layer
                DISPLAY_FORMAT "04X";
                EDIT_FORMAT "04X";
            }
        }
    }
}

VARIABLE jad_junk_1
{
    LABEL "junk";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ;
    TYPE INTEGER (1);
}





#include "Phoenix_Data.dd"

// Common items for a wireless adapter (modify as needed for your particular device)

// Make sure the burst and event subdevice indicies are updated when the subdevice count changes
#if __TOKVER__ >= 800
REFRESH subdev_refresh_relation
{
    devices_detected:
//jad
/*
        burst_messages[0].SUB_DEVICE_MISSING,
        burst_messages[1].SUB_DEVICE_MISSING,
        burst_messages[2].SUB_DEVICE_MISSING,
        burst_messages[3].SUB_DEVICE_MISSING,
        burst_messages[0].SUB_DEVICE_MAPPING,
        burst_messages[1].SUB_DEVICE_MAPPING,
        burst_messages[2].SUB_DEVICE_MAPPING,
        burst_messages[3].SUB_DEVICE_MAPPING
*/
        burst_messages2[0].SUB_DEVICE_MISSING,
        burst_messages2[1].SUB_DEVICE_MISSING,
        burst_messages2[2].SUB_DEVICE_MISSING,
        burst_messages2[3].SUB_DEVICE_MISSING,
        burst_messages2[0].SUB_DEVICE_MAPPING,
        burst_messages2[1].SUB_DEVICE_MAPPING,
        burst_messages2[2].SUB_DEVICE_MAPPING,
        burst_messages2[3].SUB_DEVICE_MAPPING

#if INCLUDE_EVENTS
       ,events[0].EVENT_CONTROL.SUB_DEVICE_MISSING,
        events[1].EVENT_CONTROL.SUB_DEVICE_MISSING
        events[0].EVENT_CONTROL.SUB_DEVICE_MAPPING,
        events[1].EVENT_CONTROL.SUB_DEVICE_MAPPING
#endif
}

#else

REFRESH subdev_refresh_relation
{
    devices_detected:
        burst_message_collection.SUB_DEVICE_MISSING,
        burst_message_collection.SUB_DEVICE_MAPPING
#if INCLUDE_EVENTS
       ,event.EVENT_CONTROL.SUB_DEVICE_MISSING,
        event.EVENT_CONTROL.SUB_DEVICE_MAPPING
#endif
}
#endif

VARIABLE fl_batteryLife
{
    LABEL "Battery Life";
    HELP [digital_value_help];
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE batteryUSL
{
    LABEL [usl];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE batteryLSL
{
    LABEL [lsl];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE batteryDamping
{
    LABEL [damping_value];
    HELP [seconds_damping_value_help];
    HANDLING READ;
    CONSTANT_UNIT "s";
    TYPE FLOAT;
}

VARIABLE batteryMinSpan
{
    LABEL [minimum_span];
    HELP [minimum_span_help];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE batteryUnits
{
    LABEL "Units";
    HELP [digital_units_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        // Battery life only supports units of Days
        {53, "Days"}        // Standard table for [days] is d.  We want "Days".
    }
}

UNIT battery_relation
{
    batteryUnits:
        fl_batteryLife,
        batteryUSL,
        batteryLSL,
        batteryMinSpan
}

VARIABLE battery_data_quality
{
    LABEL [process_data_quality];
    HELP [process_data_quality_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        PROCESS_DATA_STATUS_CODES
    }
}

VARIABLE battery_limit_status
{
    LABEL [limit_status];
    HELP [limit_status_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        LIMIT_STATUS_CODES
    }
}

VARIABLE battery_family_status
{
    LABEL [family_status];
    HELP [family_status_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        // Battery life is not part of a device family
        { 0x08, [more_device_family_status_available]}
    }
}

VARIABLE battery_classification
{   
    LABEL [classification];
    HELP [classification_help];
    HANDLING READ;
    TYPE ENUMERATED 
    {
        // Battery Life is not classified
        DEVICE_VARIABLE_CLASSIFICATION_CODE(0)
    }
}

VARIABLE battery_family
{   
    LABEL [device_family];
    HELP [device_family_help];
    HANDLING READ;
    TYPE ENUMERATED 
    {
        // Battery Life is not in a family
        DEVICE_VARIABLE_FAMILY_CODE(250)
    }
}

VARIABLE battery_serialNum
{   
    LABEL [sensor_serial_number];
    HELP [sensor_serial_number_help];
    TYPE UNSIGNED_INTEGER(3); 
}

#if __TOKVER__ >= 800
VARIABLE battery_update_period 
{
    LABEL "Update Period";
    TYPE TIME_VALUE;
    HANDLING READ;
}
#endif

VARIABLE battery_write_dev_var_code
{
    LABEL "Battery write dev var code";
    HELP "";
    CLASS DEVICE & SERVICE;
    HANDLING WRITE;
    TYPE ENUMERATED
    {
        WRITE_DEVICE_VARIABLE_CODE(0),  /* Normal */
        WRITE_DEVICE_VARIABLE_CODE(1)   /* Fix Value */
    }
}

COLLECTION OF VARIABLE batteryLife
{
    LABEL "Battery Life";
    MEMBERS
    {   
        // Sensor Correction Information
        DIGITAL_VALUE,              fl_batteryLife;
        DIGITAL_UNITS,              batteryUnits;
        DATA_QUALITY,               battery_data_quality;
        LIMIT_STATUS,               battery_limit_status;
        DEVICE_FAMILY_STATUS,       battery_family_status;

        CLASSIFICATION,             battery_classification;
        DEVICE_FAMILY,              battery_family;

        UPPER_SENSOR_LIMIT,         batteryUSL;
        LOWER_SENSOR_LIMIT,         batteryLSL;
        MINIMUM_SPAN,               batteryMinSpan;
        DAMPING_VALUE,              dummy_variable_float;

        #if __TOKVER__ >= 800
        UPDATE_TIME_PERIOD,         battery_update_period;
        #endif

        SIMULATED,                  battery_write_dev_var_code;

        // Device Information
        SENSOR_SERIAL_NUMBER,       battery_serialNum;
    }
}

VARIABLE power_setup_option
{
    LABEL "Current Power Source ";
    HELP  "Selection for the External / Line power";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 2;
    TYPE ENUMERATED
    {
      { 1, "24V Power: Source Enable (1)" }, 
      { 2, "24V Power: Source Disable (2)"  }, 
      { 3, "Loop Power (3)" } 
    }
}



//#if __TOKVER__ >= 800
VARIABLE GlobalBurstPeriod
{
    LABEL "Global Burst Period";
    HELP  "";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    CONSTANT_UNIT "sec";
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 60;
    #else
    DEFAULT_VALUE 60;
    #endif
    TYPE UNSIGNED_INTEGER;
}

//#else
/*
VARIABLE update_period_int 
{
    LABEL "Update Period";
    HELP "Update period for burst message";
    CLASS DEVICE;
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
}

*/
//#endif


VARIABLE dummy_variable
{
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE dummy_variable_float
{
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE dummy_variable_unsigned_3
{
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (3);
}

#if __TOKVER__ >= 800
VARIABLE dummy_variable_time
{
    HANDLING READ;
    TYPE TIME_VALUE;
}
#endif

VARIABLE BurstModeControl
{
    LABEL "Burst Mode Control";
    HELP  "";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 2;
    #endif
    TYPE ENUMERATED
    {
        BURST_MODE_CONTROL_CODE(0),  /* Off */   
        { 2,  "On", "Burst Mode Enabled " }
    }
}



VARIABLE ProcessCommand
{
    LABEL "Process Command";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 1;
    #endif
    TYPE ENUMERATED
    {
        { 0, "Process Data Burst Message Off", "help" },
        { 1, "Command 1", "help" },
        { 2, "Command 2", "help" },
        { 3, "Command 3", "help" }

    }
}

VARIABLE StatusCommand
{
    LABEL "Burst status command";
    HELP  "";
//jad
//    CLASS HART;
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 48;
    #endif
    TYPE ENUMERATED
    {
       { 0, "Status Data Burst Message Off", "help" }, 
       { 48, "Command 48", "help" }
 
    }
  
}



/* This collection is used to point to for all the
   device variables that are really not used, but
   required to be defined so that the burst variables
   indexes are valid. Referring to any items in this
   collection is not recommended. */
COLLECTION OF VARIABLE scratchData
{
    LABEL "Scratch Data";
    MEMBERS
    {   
        // Sensor Correction Information
        DIGITAL_VALUE,              dummy_variable_float;
        DIGITAL_UNITS,              dummy_variable;
        DATA_QUALITY,               dummy_variable;
        LIMIT_STATUS,               dummy_variable;
        DEVICE_FAMILY_STATUS,       dummy_variable;

        CLASSIFICATION,             dummy_variable;
        DEVICE_FAMILY,              dummy_variable;

        UPPER_SENSOR_LIMIT,         dummy_variable_float;
        LOWER_SENSOR_LIMIT,         dummy_variable_float;
        MINIMUM_SPAN,               dummy_variable_float;
        DAMPING_VALUE,              dummy_variable_float;

        #if __TOKVER__ >= 800
        UPDATE_TIME_PERIOD,         dummy_variable_time;
        #endif

        SIMULATED,                  dummy_variable;

        // Device Information
        SENSOR_SERIAL_NUMBER,       dummy_variable_unsigned_3;
    }
}

// Now model any device specific information contained in your adapter

/* 
 * This is the minimum set of members that any device variable collection must have
 * Define these for each device variable contained in your device, and then add to
 * the deviceVariables array.
 */
VARIABLE var0_value
{
    LABEL "Dev Var 0";
    HELP [digital_value_help];
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE var0_usl
{
    LABEL [usl];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE var0_lsl
{
    LABEL [lsl];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE var0_damping
{
    LABEL [damping_value];
    HELP [seconds_damping_value_help];
    HANDLING READ;
    CONSTANT_UNIT "s";
    TYPE FLOAT;
}

VARIABLE var0_minSpan
{
    LABEL [minimum_span];
    HELP [minimum_span_help];
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE var0_units
{
    LABEL "Units";
    HELP [digital_units_help];
    TYPE ENUMERATED
    {
        // List Units code that apply to your device variable
        UNITS_CODE(32),
        UNITS_CODE(33)
    }
}

UNIT var0_relation
{
    var0_units:
        var0_value,
        var0_usl,
        var0_lsl,
        var0_minSpan
}

VARIABLE var0_data_quality
{
    LABEL [process_data_quality];
    HELP [process_data_quality_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        PROCESS_DATA_STATUS_CODES
    }
}

VARIABLE var0_limit_status
{
    LABEL [limit_status];
    HELP [limit_status_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        LIMIT_STATUS_CODES
    }
}

VARIABLE var0_family_status
{
    LABEL [family_status];
    HELP [family_status_help];
    HANDLING READ;
    TYPE ENUMERATED
    {
        // List applicable family status if appropriate
        { 0x08, [more_device_family_status_available]}
    }
}

VARIABLE var0_classification
{   
    LABEL [classification];
    HELP [classification_help];
    HANDLING READ;
    TYPE ENUMERATED 
    {
        // List the proper classification code
        DEVICE_VARIABLE_CLASSIFICATION_CODE(0)
    }
}

VARIABLE var0_family
{   
    LABEL [device_family];
    HELP [device_family_help];
    HANDLING READ;
    TYPE ENUMERATED 
    {
        // List proper family code
        DEVICE_VARIABLE_FAMILY_CODE(250)
    }
}

VARIABLE var0_serialNum
{   
    LABEL [sensor_serial_number];
    HELP [sensor_serial_number_help];
    TYPE UNSIGNED_INTEGER(3); 
}

#if __TOKVER__ >= 800
VARIABLE var0_update_period 
{
    LABEL "Update Period";
    TYPE TIME_VALUE;
    HANDLING READ;
}
#endif

VARIABLE var0_write_dev_var_code
{
    LABEL "Dev Var 0 write dev var code";
    HELP "";
    CLASS DEVICE & SERVICE;
    HANDLING WRITE;
    TYPE ENUMERATED
    {
        WRITE_DEVICE_VARIABLE_CODE(0),  /* Normal */
        WRITE_DEVICE_VARIABLE_CODE(1)   /* Fix Value */
    }
}

COLLECTION OF VARIABLE devVar0
{
    LABEL "Device Var 0";
    MEMBERS
    {   
        // Sensor Correction Information
        DIGITAL_VALUE,              var0_value;
        DIGITAL_UNITS,              var0_units;
        DATA_QUALITY,               var0_data_quality;
        LIMIT_STATUS,               var0_limit_status;
        DEVICE_FAMILY_STATUS,       var0_family_status;

        CLASSIFICATION,             var0_classification;
        DEVICE_FAMILY,              var0_family;

        UPPER_SENSOR_LIMIT,         var0_usl;
        LOWER_SENSOR_LIMIT,         var0_lsl;
        MINIMUM_SPAN,               var0_minSpan;
        DAMPING_VALUE,              var0_damping;

        #if __TOKVER__ >= 800
        UPDATE_TIME_PERIOD,         var0_update_period;
        #endif

        SIMULATED,                  var0_write_dev_var_code;

        // Device Information
        SENSOR_SERIAL_NUMBER,       var0_serialNum;
    }
}


/* These are the Device Variables that can possibly be tied to PV, SV, TV, & QV.
   They cannot be made a part of the deviceVariables array because it causes
   a recursive definition which causes older versions of the tokenizer to
   crash when generating the ptoc table. */
ARRAY OF COLLECTION mappableVariables 
{
    ELEMENTS
    {
        0, devVar0,               "Device Variable 0";
        1, scratchData,           "Device Variable 1";
        2, scratchData,           "Device Variable 2";
        3, scratchData,           "Device Variable 3";
        4, scratchData,           "Device Variable 4";
        5, scratchData,           "Device Variable 5";
        6, scratchData,           "Device Variable 6";
        7, scratchData,           "Device Variable 7";
        8, scratchData,           "Device Variable 8";
        9, scratchData,           "Device Variable 9";
        10, scratchData,          "Device Variable 10";
        11, scratchData,          "Device Variable 11";
        12, scratchData,          "Device Variable 12";
        13, scratchData,          "Device Variable 13";
        14, scratchData,          "Device Variable 14";
        15, scratchData,          "Device Variable 15";
        16, scratchData,          "Device Variable 16";
        17, scratchData,          "Device Variable 17";
        18, scratchData,          "Device Variable 18";
        19, scratchData,          "Device Variable 19";
        20, scratchData,          "Device Variable 20";
        21, scratchData,          "Device Variable 21";
        22, scratchData,          "Device Variable 22";
        23, scratchData,          "Device Variable 23";
        24, scratchData,          "Device Variable 24";
        25, scratchData,          "Device Variable 25";
        26, scratchData,          "Device Variable 26";
        27, scratchData,          "Device Variable 27";
        28, scratchData,          "Device Variable 28";
        29, scratchData,          "Device Variable 29";
        30, scratchData,          "Device Variable 30";
        31, scratchData,          "Device Variable 31";
        32, scratchData,          "Device Variable 32";
        33, scratchData,          "Device Variable 33";
        34, scratchData,          "Device Variable 34";
        35, scratchData,          "Device Variable 35";
        36, scratchData,          "Device Variable 36";
        37, scratchData,          "Device Variable 37";
        38, scratchData,          "Device Variable 38";
        39, scratchData,          "Device Variable 39";
        40, scratchData,          "Device Variable 40";
        41, scratchData,          "Device Variable 41";
        42, scratchData,          "Device Variable 42";
        43, scratchData,          "Device Variable 43";
        44, scratchData,          "Device Variable 44";
        45, scratchData,          "Device Variable 45";
        46, scratchData,          "Device Variable 46";
        47, scratchData,          "Device Variable 47";
        48, scratchData,          "Device Variable 48";
        49, scratchData,          "Device Variable 49";
        50, scratchData,          "Device Variable 50";
        51, scratchData,          "Device Variable 51";
        52, scratchData,          "Device Variable 52";
        53, scratchData,          "Device Variable 53";
        54, scratchData,          "Device Variable 54";
        55, scratchData,          "Device Variable 55";
        56, scratchData,          "Device Variable 56";
        57, scratchData,          "Device Variable 57";
        58, scratchData,          "Device Variable 58";
        59, scratchData,          "Device Variable 59";
        60, scratchData,          "Device Variable 60";
        61, scratchData,          "Device Variable 61";
        62,  scratchData,         "Device Variable 62";
        63,  scratchData,         "Device Variable 63";
        64,  scratchData,         "Device Variable 64";
        65,  scratchData,         "Device Variable 65";
        66,  scratchData,         "Device Variable 66";
        67,  scratchData,         "Device Variable 67";
        68,  scratchData,         "Device Variable 68";
        69,  scratchData,         "Device Variable 69";
        70,  scratchData,         "Device Variable 70";
        71,  scratchData,         "Device Variable 71";
        72,  scratchData,         "Device Variable 72";
        73,  scratchData,         "Device Variable 73";
        74,  scratchData,         "Device Variable 74";
        75,  scratchData,         "Device Variable 75";
        76,  scratchData,         "Device Variable 76";
        77,  scratchData,         "Device Variable 77";
        78,  scratchData,         "Device Variable 78";
        79,  scratchData,         "Device Variable 79";
        80,  scratchData,         "Device Variable 80";
        81,  scratchData,         "Device Variable 81";
        82,  scratchData,         "Device Variable 82";
        83,  scratchData,         "Device Variable 83";
        84,  scratchData,         "Device Variable 84";
        85,  scratchData,         "Device Variable 85";
        86,  scratchData,         "Device Variable 86";
        87,  scratchData,         "Device Variable 87";
        88,  scratchData,         "Device Variable 88";
        89,  scratchData,         "Device Variable 89";
        90,  scratchData,         "Device Variable 90";
        91,  scratchData,         "Device Variable 91";
        92,  scratchData,         "Device Variable 92";
        93,  scratchData,         "Device Variable 93";
        94,  scratchData,         "Device Variable 94";
        95,  scratchData,         "Device Variable 95";
        96,  scratchData,         "Device Variable 96";
        97,  scratchData,         "Device Variable 97";
        98,  scratchData,         "Device Variable 98";
        99,  scratchData,         "Device Variable 99";
        100, scratchData,         "Device Variable 100";
        101, scratchData,         "Device Variable 101";
        102, scratchData,         "Device Variable 102";
        103, scratchData,         "Device Variable 103";
        104, scratchData,         "Device Variable 104";
        105, scratchData,         "Device Variable 105";
        106, scratchData,         "Device Variable 106";
        107, scratchData,         "Device Variable 107";
        108, scratchData,         "Device Variable 108";
        109, scratchData,         "Device Variable 109";
        110, scratchData,         "Device Variable 110";
        111, scratchData,         "Device Variable 111";
        112, scratchData,         "Device Variable 112";
        113, scratchData,         "Device Variable 113";
        114, scratchData,         "Device Variable 114";
        115, scratchData,         "Device Variable 115";
        116, scratchData,         "Device Variable 116";
        117, scratchData,         "Device Variable 117";
        118, scratchData,         "Device Variable 118";
        119, scratchData,         "Device Variable 119";
        120, scratchData,         "Device Variable 120";
        121, scratchData,         "Device Variable 121";
        122, scratchData,         "Device Variable 122";
        123, scratchData,         "Device Variable 123";
        124, scratchData,         "Device Variable 124";
        125, scratchData,         "Device Variable 125";
        126, scratchData,         "Device Variable 126";
        127, scratchData,         "Device Variable 127";
        128, scratchData,         "Device Variable 128";
        129, scratchData,         "Device Variable 129";
        130, scratchData,         "Device Variable 130";
        131, scratchData,         "Device Variable 131";
        132, scratchData,         "Device Variable 132";
        133, scratchData,         "Device Variable 133";
        134, scratchData,         "Device Variable 134";
        135, scratchData,         "Device Variable 135";
        136, scratchData,         "Device Variable 136";
        137, scratchData,         "Device Variable 137";
        138, scratchData,         "Device Variable 138";
        139, scratchData,         "Device Variable 139";
        140, scratchData,         "Device Variable 140";
        141, scratchData,         "Device Variable 141";
        142, scratchData,         "Device Variable 142";
        143, scratchData,         "Device Variable 143";
        144, scratchData,         "Device Variable 144";
        145, scratchData,         "Device Variable 145";
        146, scratchData,         "Device Variable 146";
        147, scratchData,         "Device Variable 147";
        148, scratchData,         "Device Variable 148";
        149, scratchData,         "Device Variable 149";
        150, scratchData,         "Device Variable 150";
        151, scratchData,         "Device Variable 151";
        152, scratchData,         "Device Variable 152";
        153, scratchData,         "Device Variable 153";
        154, scratchData,         "Device Variable 154";
        155, scratchData,         "Device Variable 155";
        156, scratchData,         "Device Variable 156";
        157, scratchData,         "Device Variable 157";
        158, scratchData,         "Device Variable 158";
        159, scratchData,         "Device Variable 159";
        160, scratchData,         "Device Variable 160";
        161, scratchData,         "Device Variable 161";
        162, scratchData,         "Device Variable 162";
        163, scratchData,         "Device Variable 163";
        164, scratchData,         "Device Variable 164";
        165, scratchData,         "Device Variable 165";
        166, scratchData,         "Device Variable 166";
        167, scratchData,         "Device Variable 167";
        168, scratchData,         "Device Variable 168";
        169, scratchData,         "Device Variable 169";
        170, scratchData,         "Device Variable 170";
        171, scratchData,         "Device Variable 171";
        172, scratchData,         "Device Variable 172";
        173, scratchData,         "Device Variable 173";
        174, scratchData,         "Device Variable 174";
        175, scratchData,         "Device Variable 175";
        176, scratchData,         "Device Variable 176";
        177, scratchData,         "Device Variable 177";
        178, scratchData,         "Device Variable 178";
        179, scratchData,         "Device Variable 179";
        180, scratchData,         "Device Variable 180";
        181, scratchData,         "Device Variable 181";
        182, scratchData,         "Device Variable 182";
        183, scratchData,         "Device Variable 183";
        184, scratchData,         "Device Variable 184";
        185, scratchData,         "Device Variable 185";
        186, scratchData,         "Device Variable 186";
        187, scratchData,         "Device Variable 187";
        188, scratchData,         "Device Variable 188";
        189, scratchData,         "Device Variable 189";
        190, scratchData,         "Device Variable 190";
        191, scratchData,         "Device Variable 191";
        192, scratchData,         "Device Variable 192";
        193, scratchData,         "Device Variable 193";
        194, scratchData,         "Device Variable 194";
        195, scratchData,         "Device Variable 195";
        196, scratchData,         "Device Variable 196";
        197, scratchData,         "Device Variable 197";
        198, scratchData,         "Device Variable 198";
        199, scratchData,         "Device Variable 199";
        200, scratchData,         "Device Variable 200";
        201, scratchData,         "Device Variable 201";
        202, scratchData,         "Device Variable 202";
        203, scratchData,         "Device Variable 203";
        204, scratchData,         "Device Variable 204";
        205, scratchData,         "Device Variable 205";
        206, scratchData,         "Device Variable 206";
#if __TOKVER__ >= 800
        207, scratchData,         "Device Variable 207";
        208, scratchData,         "Device Variable 208";
        209, scratchData,         "Device Variable 209";
        210, scratchData,         "Device Variable 210";
        211, scratchData,         "Device Variable 211";
        212, scratchData,         "Device Variable 212";
        213, scratchData,         "Device Variable 213";
        214, scratchData,         "Device Variable 214";
        215, scratchData,         "Device Variable 215";
        216, scratchData,         "Device Variable 216";
        217, scratchData,         "Device Variable 217";
        218, scratchData,         "Device Variable 218";
        219, scratchData,         "Device Variable 219";
        220, scratchData,         "Device Variable 220";
        221, scratchData,         "Device Variable 221";
        222, scratchData,         "Device Variable 222";
        223, scratchData,         "Device Variable 223";
        224, scratchData,         "Device Variable 224";
        225, scratchData,         "Device Variable 225";
        226, scratchData,         "Device Variable 226";
        227, scratchData,         "Device Variable 227";
        228, scratchData,         "Device Variable 228";
        229, scratchData,         "Device Variable 229";
        230, scratchData,         "Device Variable 230";
        231, scratchData,         "Device Variable 231";
        232, scratchData,         "Device Variable 232";
        233, scratchData,         "Device Variable 233";
        234, scratchData,         "Device Variable 234";
        235, scratchData,         "Device Variable 235";
        236, scratchData,         "Device Variable 236";
        237, scratchData,         "Device Variable 237";
        238, scratchData,         "Device Variable 238";
        239, scratchData,         "Device Variable 239";
        240, scratchData,         "Device Variable 240";
        241, scratchData,         "Device Variable 241";
        242, scratchData,         "Device Variable 242";
#endif
        243, batteryLife,         device_variable_code_codes(243);
    }
}
/* Because IO Adapters do not know what device variables are
   supported in the subdevice that it will be attached to,
   all entries from 0-249 need to be declared so that all
   possible burst variable codes will be valid. For the
   variable indexes not used for this device, the other
   indexes will point to a global data holder to save size. */
/* Variables 243-249 are predefined variables that should be
   supported if applicable to the device. */
/* This Array of device variables includes the common defined HCF variables.
   They cannot be made a part of the deviceVariables array because it causes
   a recursive definition which causes older versions of the tokenizer to
   crash when generating the ptoc table. */ 
ARRAY OF COLLECTION deviceVariables 
{
    ELEMENTS
    {
        0, devVar0,               "Device Variable 0";    // Shouldn't use names specific to this device because these could be for subdevices
        1, scratchData,           "Device Variable 1";
        2, scratchData,           "Device Variable 2";
        3, scratchData,           "Device Variable 3";
        4, scratchData,           "Device Variable 4";
        5, scratchData,           "Device Variable 5";
        6, scratchData,           "Device Variable 6";
        7, scratchData,           "Device Variable 7";
        8, scratchData,           "Device Variable 8";
        9, scratchData,           "Device Variable 9";
        10, scratchData,          "Device Variable 10";
        11, scratchData,          "Device Variable 11";
        12, scratchData,          "Device Variable 12";
        13, scratchData,          "Device Variable 13";
        14, scratchData,          "Device Variable 14";
        15, scratchData,          "Device Variable 15";
        16, scratchData,          "Device Variable 16";
        17, scratchData,          "Device Variable 17";
        18, scratchData,          "Device Variable 18";
        19, scratchData,          "Device Variable 19";
        20, scratchData,          "Device Variable 20";
        21, scratchData,          "Device Variable 21";
        22, scratchData,          "Device Variable 22";
        23, scratchData,          "Device Variable 23";
        24, scratchData,          "Device Variable 24";
        25, scratchData,          "Device Variable 25";
        26, scratchData,          "Device Variable 26";
        27, scratchData,          "Device Variable 27";
        28, scratchData,          "Device Variable 28";
        29, scratchData,          "Device Variable 29";
        30, scratchData,          "Device Variable 30";
        31, scratchData,          "Device Variable 31";
        32, scratchData,          "Device Variable 32";
        33, scratchData,          "Device Variable 33";
        34, scratchData,          "Device Variable 34";
        35, scratchData,          "Device Variable 35";
        36, scratchData,          "Device Variable 36";
        37, scratchData,          "Device Variable 37";
        38, scratchData,          "Device Variable 38";
        39, scratchData,          "Device Variable 39";
        40, scratchData,          "Device Variable 40";
        41, scratchData,          "Device Variable 41";
        42, scratchData,          "Device Variable 42";
        43, scratchData,          "Device Variable 43";
        44, scratchData,          "Device Variable 44";
        45, scratchData,          "Device Variable 45";
        46, scratchData,          "Device Variable 46";
        47, scratchData,          "Device Variable 47";
        48, scratchData,          "Device Variable 48";
        49, scratchData,          "Device Variable 49";
        50, scratchData,          "Device Variable 50";
        51, scratchData,          "Device Variable 51";
        52, scratchData,          "Device Variable 52";
        53, scratchData,          "Device Variable 53";
        54, scratchData,          "Device Variable 54";
        55, scratchData,          "Device Variable 55";
        56, scratchData,          "Device Variable 56";
        57, scratchData,          "Device Variable 57";
        58, scratchData,          "Device Variable 58";
        59, scratchData,          "Device Variable 59";
        60, scratchData,          "Device Variable 60";
        61, scratchData,          "Device Variable 61";
        62,  scratchData,         "Device Variable 62";
        63,  scratchData,         "Device Variable 63";
        64,  scratchData,         "Device Variable 64";
        65,  scratchData,         "Device Variable 65";
        66,  scratchData,         "Device Variable 66";
        67,  scratchData,         "Device Variable 67";
        68,  scratchData,         "Device Variable 68";
        69,  scratchData,         "Device Variable 69";
        70,  scratchData,         "Device Variable 70";
        71,  scratchData,         "Device Variable 71";
        72,  scratchData,         "Device Variable 72";
        73,  scratchData,         "Device Variable 73";
        74,  scratchData,         "Device Variable 74";
        75,  scratchData,         "Device Variable 75";
        76,  scratchData,         "Device Variable 76";
        77,  scratchData,         "Device Variable 77";
        78,  scratchData,         "Device Variable 78";
        79,  scratchData,         "Device Variable 79";
        80,  scratchData,         "Device Variable 80";
        81,  scratchData,         "Device Variable 81";
        82,  scratchData,         "Device Variable 82";
        83,  scratchData,         "Device Variable 83";
        84,  scratchData,         "Device Variable 84";
        85,  scratchData,         "Device Variable 85";
        86,  scratchData,         "Device Variable 86";
        87,  scratchData,         "Device Variable 87";
        88,  scratchData,         "Device Variable 88";
        89,  scratchData,         "Device Variable 89";
        90,  scratchData,         "Device Variable 90";
        91,  scratchData,         "Device Variable 91";
        92,  scratchData,         "Device Variable 92";
        93,  scratchData,         "Device Variable 93";
        94,  scratchData,         "Device Variable 94";
        95,  scratchData,         "Device Variable 95";
        96,  scratchData,         "Device Variable 96";
        97,  scratchData,         "Device Variable 97";
        98,  scratchData,         "Device Variable 98";
        99,  scratchData,         "Device Variable 99";
        100, scratchData,         "Device Variable 100";
        101, scratchData,         "Device Variable 101";
        102, scratchData,         "Device Variable 102";
        103, scratchData,         "Device Variable 103";
        104, scratchData,         "Device Variable 104";
        105, scratchData,         "Device Variable 105";
        106, scratchData,         "Device Variable 106";
        107, scratchData,         "Device Variable 107";
        108, scratchData,         "Device Variable 108";
        109, scratchData,         "Device Variable 109";
        110, scratchData,         "Device Variable 110";
        111, scratchData,         "Device Variable 111";
        112, scratchData,         "Device Variable 112";
        113, scratchData,         "Device Variable 113";
        114, scratchData,         "Device Variable 114";
        115, scratchData,         "Device Variable 115";
        116, scratchData,         "Device Variable 116";
        117, scratchData,         "Device Variable 117";
        118, scratchData,         "Device Variable 118";
        119, scratchData,         "Device Variable 119";
        120, scratchData,         "Device Variable 120";
        121, scratchData,         "Device Variable 121";
        122, scratchData,         "Device Variable 122";
        123, scratchData,         "Device Variable 123";
        124, scratchData,         "Device Variable 124";
        125, scratchData,         "Device Variable 125";
        126, scratchData,         "Device Variable 126";
        127, scratchData,         "Device Variable 127";
        128, scratchData,         "Device Variable 128";
        129, scratchData,         "Device Variable 129";
        130, scratchData,         "Device Variable 130";
        131, scratchData,         "Device Variable 131";
        132, scratchData,         "Device Variable 132";
        133, scratchData,         "Device Variable 133";
        134, scratchData,         "Device Variable 134";
        135, scratchData,         "Device Variable 135";
        136, scratchData,         "Device Variable 136";
        137, scratchData,         "Device Variable 137";
        138, scratchData,         "Device Variable 138";
        139, scratchData,         "Device Variable 139";
        140, scratchData,         "Device Variable 140";
        141, scratchData,         "Device Variable 141";
        142, scratchData,         "Device Variable 142";
        143, scratchData,         "Device Variable 143";
        144, scratchData,         "Device Variable 144";
        145, scratchData,         "Device Variable 145";
        146, scratchData,         "Device Variable 146";
        147, scratchData,         "Device Variable 147";
        148, scratchData,         "Device Variable 148";
        149, scratchData,         "Device Variable 149";
        150, scratchData,         "Device Variable 150";
        151, scratchData,         "Device Variable 151";
        152, scratchData,         "Device Variable 152";
        153, scratchData,         "Device Variable 153";
        154, scratchData,         "Device Variable 154";
        155, scratchData,         "Device Variable 155";
        156, scratchData,         "Device Variable 156";
        157, scratchData,         "Device Variable 157";
        158, scratchData,         "Device Variable 158";
        159, scratchData,         "Device Variable 159";
        160, scratchData,         "Device Variable 160";
        161, scratchData,         "Device Variable 161";
        162, scratchData,         "Device Variable 162";
        163, scratchData,         "Device Variable 163";
        164, scratchData,         "Device Variable 164";
        165, scratchData,         "Device Variable 165";
        166, scratchData,         "Device Variable 166";
        167, scratchData,         "Device Variable 167";
        168, scratchData,         "Device Variable 168";
        169, scratchData,         "Device Variable 169";
        170, scratchData,         "Device Variable 170";
        171, scratchData,         "Device Variable 171";
        172, scratchData,         "Device Variable 172";
        173, scratchData,         "Device Variable 173";
        174, scratchData,         "Device Variable 174";
        175, scratchData,         "Device Variable 175";
        176, scratchData,         "Device Variable 176";
        177, scratchData,         "Device Variable 177";
        178, scratchData,         "Device Variable 178";
        179, scratchData,         "Device Variable 179";
        180, scratchData,         "Device Variable 180";
        181, scratchData,         "Device Variable 181";
        182, scratchData,         "Device Variable 182";
        183, scratchData,         "Device Variable 183";
        184, scratchData,         "Device Variable 184";
        185, scratchData,         "Device Variable 185";
        186, scratchData,         "Device Variable 186";
        187, scratchData,         "Device Variable 187";
        188, scratchData,         "Device Variable 188";
        189, scratchData,         "Device Variable 189";
        190, scratchData,         "Device Variable 190";
        191, scratchData,         "Device Variable 191";
        192, scratchData,         "Device Variable 192";
        193, scratchData,         "Device Variable 193";
        194, scratchData,         "Device Variable 194";
        195, scratchData,         "Device Variable 195";
        196, scratchData,         "Device Variable 196";
        197, scratchData,         "Device Variable 197";
        198, scratchData,         "Device Variable 198";
        199, scratchData,         "Device Variable 199";
        200, scratchData,         "Device Variable 200";
        201, scratchData,         "Device Variable 201";
        202, scratchData,         "Device Variable 202";
        203, scratchData,         "Device Variable 203";
        204, scratchData,         "Device Variable 204";
        205, scratchData,         "Device Variable 205";
        206, scratchData,         "Device Variable 206";
#if __TOKVER__ >= 800
        207, scratchData,         "Device Variable 207";
        208, scratchData,         "Device Variable 208";
        209, scratchData,         "Device Variable 209";
        210, scratchData,         "Device Variable 210";
        211, scratchData,         "Device Variable 211";
        212, scratchData,         "Device Variable 212";
        213, scratchData,         "Device Variable 213";
        214, scratchData,         "Device Variable 214";
        215, scratchData,         "Device Variable 215";
        216, scratchData,         "Device Variable 216";
        217, scratchData,         "Device Variable 217";
        218, scratchData,         "Device Variable 218";
        219, scratchData,         "Device Variable 219";
        220, scratchData,         "Device Variable 220";
        221, scratchData,         "Device Variable 221";
        222, scratchData,         "Device Variable 222";
        223, scratchData,         "Device Variable 223";
        224, scratchData,         "Device Variable 224";
        225, scratchData,         "Device Variable 225";
        226, scratchData,         "Device Variable 226";
        227, scratchData,         "Device Variable 227";
        228, scratchData,         "Device Variable 228";
        229, scratchData,         "Device Variable 229";
        230, scratchData,         "Device Variable 230";
        231, scratchData,         "Device Variable 231";
        232, scratchData,         "Device Variable 232";
        233, scratchData,         "Device Variable 233";
        234, scratchData,         "Device Variable 234";
        235, scratchData,         "Device Variable 235";
        236, scratchData,         "Device Variable 236";
        237, scratchData,         "Device Variable 237";
        238, scratchData,         "Device Variable 238";
        239, scratchData,         "Device Variable 239";
        240, scratchData,         "Device Variable 240";
        241, scratchData,         "Device Variable 241";
        242, scratchData,         "Device Variable 242";
#endif
        243, batteryLife,         device_variable_code_codes(243);
        244, scratchData,         device_variable_code_codes(244);
        245, scratchData,         device_variable_code_codes(245);
        246, PV.DEVICE_VARIABLE,  device_variable_code_codes(246);
        247, scratchData,         device_variable_code_codes(247);
        248, scratchData,         device_variable_code_codes(248);
        249, scratchData,         device_variable_code_codes(249);
        250, scratchData,         [not_used];
    }
}

#if __TOKVER__ >= 800
//jad
/*
UNIT burst_trigger0_units_relation
{
    burst_messages[0].BURST_UNIT:
    burst_messages[0].TRIGGER_LEVEL
}
UNIT burst_trigger1_units_relation
{
    burst_messages[1].BURST_UNIT:
    burst_messages[1].TRIGGER_LEVEL
}
UNIT burst_trigger2_units_relation
{
    burst_messages[2].BURST_UNIT:
    burst_messages[2].TRIGGER_LEVEL
}
UNIT burst_trigger3_units_relation
{
    burst_messages[3].BURST_UNIT:
    burst_messages[3].TRIGGER_LEVEL
}
*/
UNIT burst_trigger0_units_relation
{
    burst_messages2[0].BURST_UNIT:
        burst_messages2[0].TRIGGER_LEVEL
}
UNIT burst_trigger1_units_relation
{
    burst_messages2[1].BURST_UNIT:
        burst_messages2[1].TRIGGER_LEVEL
}
UNIT burst_trigger2_units_relation
{
    burst_messages2[2].BURST_UNIT:
        burst_messages2[2].TRIGGER_LEVEL
}
UNIT burst_trigger3_units_relation
{
    burst_messages2[3].BURST_UNIT:
        burst_messages2[3].TRIGGER_LEVEL
}
#else
UNIT burst_trigger0_units_relation
{
    burst_message_collection.BURST_UNIT:
        burst_message_collection.TRIGGER_LEVEL
}
#endif

/* Add other unit relations for each burst message supported in the device */

/***************************************************************************
 * below is an example menu for a wireless adapter.  Modify to fit the needs
 * of your device.
 */

MENU download_variables
{	//read
	LABEL   "Read from the device";
	ITEMS
	{
		tag,
		longTag,
		descriptor,
		message,
		final_assembly_number,
		polling_address,
		power_setup_option,
		devices_detected,
		subdev0_manufacturer_id,
		subdev0_expanded_device_type,
		subdev0_long_tag,
		burst_mode_select_0,
		burst_command_number_0,
		update_period_0
		network_id,
		//join_key_1,
		//join_key_2,
		//join_key_3,
		//join_key_4,
		loopCurrent,
		device_type,
		private_label_distributor,
		device_id,
		config_change_counter,
		max_num_device_variables,
		hardware_revision,
		software_revision,
		total_number_burst_messages,
		max_io_cards,
		max_channels_per_io_card,
		max_sub_dev_per_channel,
		max_dr,
		Join_Status
	}
}

MENU upload_variables
{	//write
	LABEL   "Write to the device";
	ITEMS
	{
		tag,
		longTag,
		descriptor,
		message,
		final_assembly_number,
		polling_address,
		burst_mode_select_0,
		burst_command_number_0,
		update_period_0,
		network_id,
		join_key_1,
		join_key_2,
		join_key_3,
		join_key_4
	}
}



MENU offline_root_menu
{
    LABEL [bus_devices];
    ITEMS
    {
        device_setup_offline    
    }
}

MENU root_menu
{
    LABEL [bus_devices];
    ITEMS
    {
        device_setup    
    }
}

MENU device_setup_offline
{
    LABEL "RAD-WHA";//[device_setup];
    ITEMS
    {
        basic_setup_offline,                                                // menu
        detailed_setup_offline,                                             // menu
        wireless_output,
        //process_variables,                                          // menu                                                                                                             // 
        review (REVIEW),
        Joining_offline       
        //PV.DAQ.ANALOG_VALUE
    }
}
MENU device_setup
{
    LABEL "RAD-WHA";//[device_setup];
    ITEMS
    {
        basic_setup,                                                // menu
        detailed_setup,                                             // menu
        wireless_output,
        process_variables,                                          // menu                                                                                                             // 
        review (REVIEW),
        Joining       
        //PV.DAQ.ANALOG_VALUE
    }
}

MENU process_variables
{
    //LABEL [process_variables];
    LABEL "Variables";
    ITEMS
    {
        PV.DAQ.ANALOG_VALUE    
        
        /* 
         * Add your device variables here
         */
       // deviceVariables[243].DIGITAL_VALUE (DISPLAY_VALUE)          // Battery life if applicable
    }
}
/*
MENU diag_service
{
    LABEL [diagnostics_and_service];
    ITEMS
    {
        test_device                                                 // menu
    }
}
*/

MENU status_display
{
    LABEL "Device status";
    ITEMS
    {
        device_status (DISPLAY_VALUE),
        extended_fld_device_status (DISPLAY_VALUE),
        standardized_status_0 (DISPLAY_VALUE),
        standardized_status_1 (DISPLAY_VALUE),
        standardized_status_2 (DISPLAY_VALUE),
        standardized_status_3 (DISPLAY_VALUE),
        device_specific_status_0 (DISPLAY_VALUE),
        device_specific_status_1 (DISPLAY_VALUE),
        device_specific_status_2 (DISPLAY_VALUE),
        device_specific_status_3 (DISPLAY_VALUE),
        device_specific_status_4 (DISPLAY_VALUE),
        device_specific_status_5 (DISPLAY_VALUE),

    /*
     * Add any additional device specific status here
     */
        config_change_counter (DISPLAY_VALUE),

        message_statistics                                          // menu

    }
}

MENU basic_setup_offline
{
   // LABEL [basic_setup];
     LABEL "General Setup";
    ITEMS
    {
        tag (DISPLAY_VALUE),
        longTag (DISPLAY_VALUE),
        descriptor (DISPLAY_VALUE),
        message (DISPLAY_VALUE),        
        final_assembly_number (DISPLAY_VALUE),
        polling_address (DISPLAY_VALUE),
        power_setup_option(DISPLAY_VALUE,READ_ONLY)
    }
}
MENU basic_setup
{
   // LABEL [basic_setup];
     LABEL "General Setup";
    ITEMS
    {
        tag (DISPLAY_VALUE),
        longTag (DISPLAY_VALUE),
        descriptor (DISPLAY_VALUE),
        message (DISPLAY_VALUE),        
        final_assembly_number (DISPLAY_VALUE),
        polling_address (DISPLAY_VALUE),
        power_setup_option(DISPLAY_VALUE,READ_ONLY),
        Power
                                      // menu
    }
}

MENU detailed_setup_offline
{
    //LABEL [detailed_setup];
    LABEL "Sub-device Setup";
    ITEMS
    {
         subdevice_info_offline,
         burst_config_offline           
    }
}
MENU detailed_setup
{
    //LABEL [detailed_setup];
    LABEL "Sub-device Setup";
    ITEMS
    {
         subdevice_info,
         burst_config           
    }
}

MENU device_info
{
    LABEL [device_information];
    ITEMS
    {
        private_label_distributor (DISPLAY_VALUE),
        device_type (DISPLAY_VALUE),
        device_id (DISPLAY_VALUE),
        config_change_counter (DISPLAY_VALUE),
        tag (DISPLAY_VALUE),
        longTag (DISPLAY_VALUE),
        //date (DISPLAY_VALUE),
        write_protect (DISPLAY_VALUE),
        descriptor (DISPLAY_VALUE),
        message (DISPLAY_VALUE),
        final_assembly_number (DISPLAY_VALUE),
        power_setup_option (DISPLAY_VALUE,READ_ONLY),
        device_flags (DISPLAY_VALUE),
        device_profile (DISPLAY_VALUE),
        device_capabilities,                                        // menu
        device_revisions                                            // menu
    }
}

MENU device_revisions
{
    LABEL [revision_numbers];
    ITEMS
    {
        universal_revision (DISPLAY_VALUE),
        transmitter_revision (DISPLAY_VALUE),
        software_revision (DISPLAY_VALUE),
        hardware_revision (DISPLAY_VALUE)
    }
}

MENU output_conditioning
{
    LABEL "Output Configuration";
    ITEMS
    {
        hart_output                                                // menu        
    }
}

MENU hart_output
{
    LABEL "HART";
    ITEMS
    {
        polling_address (DISPLAY_VALUE),
        request_preambles (DISPLAY_VALUE),
        response_preambles (DISPLAY_VALUE),
        master_mode (DISPLAY_VALUE,READ_ONLY),
        retry_count (DISPLAY_VALUE,READ_ONLY),
        burst_config       
#if (__TOKVER__ >= 800) && (INCLUDE_EVENTS)
       ,event_config                                                // menu
#endif
    }
}

MENU review
{
    LABEL "Information";//[review];
    ITEMS
    {
        device_type (DISPLAY_VALUE, READ_ONLY),
        private_label_distributor (DISPLAY_VALUE, READ_ONLY),        
        device_id (DISPLAY_VALUE, READ_ONLY),        
        config_change_counter (DISPLAY_VALUE, READ_ONLY),
        max_num_device_variables  (DISPLAY_VALUE, READ_ONLY),
        hardware_revision (DISPLAY_VALUE, READ_ONLY),
        software_revision (DISPLAY_VALUE, READ_ONLY),
        total_number_burst_messages (DISPLAY_VALUE,READ_ONLY),
        max_io_cards (DISPLAY_VALUE, READ_ONLY),
        max_channels_per_io_card (DISPLAY_VALUE, READ_ONLY),
        max_sub_dev_per_channel (DISPLAY_VALUE, READ_ONLY),
        max_dr (DISPLAY_VALUE, READ_ONLY)
                
    }
}


#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
MENU event_status_menu
{
    LABEL "Event status";
    ITEMS
    {
        event1_status,
        event2_status
    }
}
#endif

#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
MENU event1_status
{
    LABEL "Event 1";
    ITEMS
    {
#if __TOKVER__ >= 800
//        subdevice_list[events[0].EVENT_CONTROL.SUB_DEVICE_MAPPING].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.SUB_DEVICE_MISSING (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.EVENT_STATUS (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.TIME_OF_FIRST_UNACK_EVENT (DISPLAY_VALUE)
#else
        event.EVENT_CONTROL.EVENT_STATUS (DISPLAY_VALUE),
        event.EVENT_CONTROL.TIME_OF_FIRST_UNACK_EVENT (DISPLAY_VALUE)
#endif
    }
}
#endif

MENU device_capabilities
{
    LABEL "Device capabilities";
    ITEMS
    {
        total_number_burst_messages (DISPLAY_VALUE),
#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
        number_events_supported (DISPLAY_VALUE),
#endif
        max_io_cards (DISPLAY_VALUE),
        max_channels_per_io_card (DISPLAY_VALUE),
        max_sub_dev_per_channel (DISPLAY_VALUE),
        max_dr (DISPLAY_VALUE)
    }
}

#if 0
MENU wireless_info
{
    LABEL "Wireless info";
    ITEMS
    {
        /*wireless_module_manufacturer_id (DISPLAY_VALUE),*/        // Optional
        /*wireless_module_device_type (DISPLAY_VALUE),*/            // Optional
        /*wireless_module_transmitter_revision (DISPLAY_VALUE),*/   // Optional
        /*wireless_module_software_revision (DISPLAY_VALUE),*/      // Optional
        /*wireless_module_hardware_revision (DISPLAY_VALUE),*/      // Optional
        device_nickname_address (DISPLAY_VALUE),
#if __TOKVER__ >= 800
        join_retry_timer (DISPLAY_VALUE),
        keep_alive_min_time (DISPLAY_VALUE),
#else
        join_retry_timer_int (DISPLAY_VALUE),
        keep_alive_min_time_int (DISPLAY_VALUE),
#endif
        wireless_capabilities                                       // menu
    }
}

MENU wireless_capabilities
{
    LABEL "Wireless capabilities";
    ITEMS
    {
        max_neighbors (DISPLAY_VALUE),
        max_packet_buffers (DISPLAY_VALUE),
        receive_signal_level (DISPLAY_VALUE),
#if __TOKVER__ >= 800
        recover_time (DISPLAY_VALUE),
        peak_packet_load (DISPLAY_VALUE),
#else
        recover_time_int (DISPLAY_VALUE),
        peak_packet_load_int (DISPLAY_VALUE),
#endif
        peak_packets_per_second (DISPLAY_VALUE)
    }
}

#endif
/*************************************************************************************************/
//burst below

//jad
//appended 2 to all burst_messages arrays
MENU burst_config_offline
{
    LABEL "Burst configuration";
    ITEMS
    {
        burst_messages2[0].BURST_MODE_SELECT(DISPLAY_VALUE),
        burst_messages2[0].COMMAND_NUMBER(DISPLAY_VALUE),
        burst_messages2[0].UPDATE_PERIOD(DISPLAY_VALUE),
        burst_menu_2
    }
}
MENU burst_config
{
    LABEL "Burst configuration";
    ITEMS
    {
        burst_messages2[0].BURST_MODE_SELECT(DISPLAY_VALUE),
        burst_messages2[0].COMMAND_NUMBER(DISPLAY_VALUE),
        burst_messages2[0].UPDATE_PERIOD(DISPLAY_VALUE),
        StatusCommandMethod,
        burst_menu_2
    }
}

//jad
//it doesn't look like this is ever called...
MENU burst_menu_1
{
    LABEL "Burst process command";
    ITEMS
    {
#if __TOKVER__ >= 800
        burst_messages2[0].BURST_MODE_SELECT(DISPLAY_VALUE),
        burst_messages2[0].UPDATE_PERIOD(DISPLAY_VALUE),
#else
       
#endif
        burst_messages2[0].COMMAND_NUMBER(DISPLAY_VALUE)
    }
}

MENU burst_vars1
{
    LABEL "Burst variables";
#if __TOKVER__ >= 800
    VALIDITY  IF ( ( burst_messages2[0].COMMAND_NUMBER == 33 ) ||
                   ( burst_messages2[0].COMMAND_NUMBER == 9  )  )   {TRUE;}   ELSE  {FALSE;}
#elif __TOKVER__ >= 600
    VALIDITY  IF ( ( burst_message_collection.COMMAND_NUMBER == 33 ) ||
                   ( burst_message_collection.COMMAND_NUMBER == 9  )  )   {TRUE;}   ELSE  {FALSE;}
#endif
    ITEMS
    {
#if __TOKVER__ >= 800
        burst_messages2[0].BURST_VAR_1 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_2 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_3 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_4 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_5 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_6 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_7 (DISPLAY_VALUE),
        burst_messages2[0].BURST_VAR_8 (DISPLAY_VALUE)
#else 
        burst_variable_code_1 (DISPLAY_VALUE),
        burst_variable_code_2 (DISPLAY_VALUE),
        burst_variable_code_3 (DISPLAY_VALUE),
        burst_variable_code_4 (DISPLAY_VALUE),
        burst_variable_code_5 (DISPLAY_VALUE),
        burst_variable_code_6 (DISPLAY_VALUE),
        burst_variable_code_7 (DISPLAY_VALUE),
        burst_variable_code_8 (DISPLAY_VALUE)
#endif
    }
}

#if __TOKVER__ >= 800
MENU burst_menu_2
{
    #if __TOKVER__ >= 800
    VALIDITY  IF ( ( burst_messages2[1].COMMAND_NUMBER == 48 ))   {TRUE;}   ELSE  {FALSE;}
#elif __TOKVER__ >= 600
    VALIDITY  IF ( ( burst_message_collection.COMMAND_NUMBER == 48 ))   {TRUE;}   ELSE  {FALSE;}
#endif
    LABEL "Burst status command";

    ITEMS
    {
        StatusCommand(DISPLAY_VALUE,READ_ONLY)
    }
}

MENU burst_vars2
{
    LABEL "Burst variables";
    VALIDITY  IF ( ( burst_messages2[1].COMMAND_NUMBER == 33 ) ||
                   ( burst_messages2[1].COMMAND_NUMBER == 9  )  )   {TRUE;}   ELSE  {FALSE;}
    ITEMS
    {
        burst_messages2[1].BURST_VAR_1 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_2 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_3 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_4 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_5 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_6 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_7 (DISPLAY_VALUE),
        burst_messages2[1].BURST_VAR_8 (DISPLAY_VALUE)
    }
}

#endif


#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
MENU event_config
{
    LABEL "Event configuration";
    ITEMS
    {
        event_menu_1                                               // menu
#if __TOKVER__ >= 800
       ,event_menu_2                                               // menu
#endif
    }
}

MENU event_menu_1
{
    LABEL "Event 1";
    ITEMS
    {
#if __TOKVER__ >= 800
        change_event1_subdevice,                                    // method
//        subdevice_list[events[0].EVENT_CONTROL.SUB_DEVICE_MAPPING].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.SUB_DEVICE_MISSING (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.EVENT_NOTIFICATION_CONTROL (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.EVENT_NOTIFICATION_RETRY_TIME (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.MAX_UPDATE_TIME (DISPLAY_VALUE),
        events[0].EVENT_CONTROL.EVENT_DEBOUNCE_INTERVAL (DISPLAY_VALUE),
#else
        event.EVENT_CONTROL.EVENT_NOTIFICATION_CONTROL (DISPLAY_VALUE),
        event.EVENT_CONTROL.EVENT_NOTIFICATION_RETRY_TIME (DISPLAY_VALUE),
        event.EVENT_CONTROL.MAX_UPDATE_TIME (DISPLAY_VALUE),
        event.EVENT_CONTROL.EVENT_DEBOUNCE_INTERVAL (DISPLAY_VALUE),
#endif
        event_mask_1                                                // menu
    }
}

MENU event_mask_1
{
    LABEL "Event Mask";
    ITEMS
    {
#if __TOKVER__ >= 800
        events[0].EVENT_MASK.DEVICE_STATUS_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.EXTENDED_FLD_DEVICE_STATUS_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.STANDARDIZED_STATUS_0_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.STANDARDIZED_STATUS_1_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.STANDARDIZED_STATUS_2_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.STANDARDIZED_STATUS_3_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.ANALOG_CHANNEL_SATURATED1_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEV_OPERATING_MODE_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_0_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_1_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_2_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_3_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_4_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_5_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_14_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_15_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_16_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_17_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_18_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_19_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_20_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_21_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_22_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_23_MASK (DISPLAY_VALUE),
        events[0].EVENT_MASK.DEVICE_SPECIFIC_STATUS_24_MASK (DISPLAY_VALUE)
#else
        event.EVENT_MASK.DEVICE_STATUS_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.EXTENDED_FLD_DEVICE_STATUS_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.STANDARDIZED_STATUS_0_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.STANDARDIZED_STATUS_1_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.STANDARDIZED_STATUS_2_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.STANDARDIZED_STATUS_3_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.ANALOG_CHANNEL_SATURATED1_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEV_OPERATING_MODE_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_0_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_1_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_2_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_3_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_4_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_5_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_14_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_15_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_16_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_17_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_18_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_19_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_20_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_21_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_22_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_23_MASK (DISPLAY_VALUE),
        event.EVENT_MASK.DEVICE_SPECIFIC_STATUS_24_MASK (DISPLAY_VALUE)
#endif

    }
}

#if __TOKVER__ >= 800
MENU event_menu_2
{
    LABEL "Event 2";
    ITEMS
    {
        change_event2_subdevice,
//        subdevice_list[events[1].EVENT_CONTROL.SUB_DEVICE_MAPPING].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.SUB_DEVICE_MISSING (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.EVENT_NOTIFICATION_CONTROL (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.EVENT_NOTIFICATION_RETRY_TIME (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.MAX_UPDATE_TIME (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.EVENT_DEBOUNCE_INTERVAL (DISPLAY_VALUE),
        event_mask_2
    }
}

MENU event_mask_2
{
    LABEL "Event Mask";
    ITEMS
    {
        events[1].EVENT_MASK.DEVICE_STATUS_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.EXTENDED_FLD_DEVICE_STATUS_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.STANDARDIZED_STATUS_0_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.STANDARDIZED_STATUS_1_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.STANDARDIZED_STATUS_2_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.STANDARDIZED_STATUS_3_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.ANALOG_CHANNEL_FIXED1_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.ANALOG_CHANNEL_SATURATED1_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_0_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_1_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_2_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_3_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_4_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_5_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_14_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_15_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_16_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_17_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_18_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_19_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_20_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_21_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_22_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_23_MASK (DISPLAY_VALUE),
        events[1].EVENT_MASK.DEVICE_SPECIFIC_STATUS_24_MASK (DISPLAY_VALUE)
    }
}
#endif
#endif

MENU wireless_output
{
    LABEL "WirelessHART Network Parameters";
    ITEMS
    {
        
        network_id (DISPLAY_VALUE),
		join_key_1 (DISPLAY_VALUE),
        join_key_2 (DISPLAY_VALUE),
        join_key_3 (DISPLAY_VALUE),
        join_key_4 (DISPLAY_VALUE)       

    }
}

MENU subdevice_info_offline
{
    LABEL "Sub-devices";
    ITEMS 
    {
        devices_detected (DISPLAY_VALUE)
#if __TOKVER__ >= 800
        ,sub_device_menu_0,
        sub_device_menu_1,
        sub_device_menu_2,
        sub_device_menu_3,
        sub_device_menu_4

    /*
     * Add more as needed
     */
#endif
    }

}
MENU subdevice_info
{
    LABEL "Sub-devices";
    ITEMS 
    {
        devices_detected (DISPLAY_VALUE),
        //PV.DAQ.ANALOG_VALUE,
        refreshSubDeviceMapping,                                     // method
        DiscoverSubDevices
#if __TOKVER__ >= 800
        ,sub_device_menu_0,
        sub_device_menu_1,
        sub_device_menu_2,
        sub_device_menu_3,
        sub_device_menu_4

    /*
     * Add more as needed
     */
#endif
    }

}

#if __TOKVER__ >= 800
MENU sub_device_menu_0
{
    LABEL "I/O adapter";
    ITEMS
    {
        subdevice_array[0].MANUFID (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[0].EXP_DEV_TYPE (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[0].LONG_TAG (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU sub_device_menu_1
{
    LABEL "Sub-device 1";
    VALIDITY  IF ( devices_detected > 0x0001 )   {TRUE;}   ELSE  {FALSE;}
    ITEMS
    {
        subdevice_array[1].MANUFID (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[1].EXP_DEV_TYPE (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[1].LONG_TAG (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU sub_device_menu_2
{
    LABEL "Sub-device 2";
    VALIDITY  IF ( devices_detected > 0x0002 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {
        subdevice_array[2].MANUFID (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[2].EXP_DEV_TYPE (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[2].LONG_TAG (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU sub_device_menu_3
{
    LABEL "Sub-device 3";
    VALIDITY  IF ( devices_detected > 0x0003 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {
        subdevice_array[3].MANUFID (DISPLAY_VALUE),
        subdevice_array[3].EXP_DEV_TYPE (DISPLAY_VALUE),
        subdevice_array[3].LONG_TAG (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU sub_device_menu_4
{
    LABEL "Sub-device 4";
    VALIDITY  IF ( devices_detected > 0x0004 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {
        subdevice_array[4].MANUFID (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[4].EXP_DEV_TYPE (DISPLAY_VALUE, READ_ONLY),
        subdevice_array[4].LONG_TAG (DISPLAY_VALUE, READ_ONLY)
    }
}

#if INCLUDE_EVENTS
MENU event2_status
{
    LABEL "Event 2";
    ITEMS
    {
//        subdevice_list[events[1].EVENT_CONTROL.SUB_DEVICE_MAPPING].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.SUB_DEVICE_MISSING (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.EVENT_STATUS (DISPLAY_VALUE),
        events[1].EVENT_CONTROL.TIME_OF_FIRST_UNACK_EVENT (DISPLAY_VALUE)
    }
}
#endif
#endif

MENU message_statistics
{
    LABEL "Message statistics";
    ITEMS
    {
       AdapterStats                  // menu
#if __TOKVER__ >= 800
       ,MasterStats,                 // menu
       subDeviceStats                // menu
#endif
    }
}

MENU AdapterStats
{
    LABEL "IO slave statistics";
    ITEMS
    {
       STX_count(DISPLAY_VALUE),
       ACK_count(DISPLAY_VALUE),
       BACK_count(DISPLAY_VALUE)
    }
}

#if __TOKVER__ >= 800
MENU MasterStats
{
    LABEL "IO master statistics";
    ITEMS
    {
        Card0Stats

    /* 
     * DD currently supports 1 IO card - add additional cards here in future as needed
     */
    }
}

MENU Card0Stats
{
    LABEL "IO card 1";
    ITEMS
    {
        Channel0Stats

    /*
     * Add menus for each additional channel supported!
     */
    }
}

/* 
 * Add more of these for each card/channel combination supported by the adapter
 */
MENU Channel0Stats
{
    LABEL "IO channel 1";
    ITEMS
    {
       io_cards[0][0].STX_SENT(DISPLAY_VALUE),
       io_cards[0][0].ACK_RECEIVED(DISPLAY_VALUE),
       io_cards[0][0].OSTX_RECEIVED(DISPLAY_VALUE),
       io_cards[0][0].OACK_RECEIVED(DISPLAY_VALUE),
       io_cards[0][0].BACK_RECEIVED(DISPLAY_VALUE)
    }
}

MENU subDeviceStats
{
    LABEL "Subdevice statistics";
    ITEMS
    {
        subdev_0_stats,             // menu
        subdev_1_stats,             // menu
        subdev_2_stats              // menu

    /*
     * Add more as needed
     */
    }
}

MENU subdev_0_stats
{
    LABEL "IO adapter";
    ITEMS
    {
//        subdevice_list[0].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        subdevice_list[0].STATS.STAT_STX_CNT (DISPLAY_VALUE),
        subdevice_list[0].STATS.STAT_ACK_CNT (DISPLAY_VALUE),
        subdevice_list[0].STATS.STAT_BACK_CNT (DISPLAY_VALUE)
    }
}

MENU subdev_1_stats
{
    LABEL "Subdevice 1";
    ITEMS
    {
//        subdevice_list[1].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        subdevice_list[1].STATS.STAT_STX_CNT (DISPLAY_VALUE),
        subdevice_list[1].STATS.STAT_ACK_CNT (DISPLAY_VALUE),
        subdevice_list[1].STATS.STAT_BACK_CNT (DISPLAY_VALUE)
    }
    VALIDITY  IF ( devices_detected > 0x0001 )   {TRUE;}   ELSE  {FALSE;}
}

MENU subdev_2_stats
{
    LABEL "Subdevice 2";
    ITEMS
    {
//        subdevice_list[2].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        subdevice_list[2].STATS.STAT_STX_CNT (DISPLAY_VALUE),
        subdevice_list[2].STATS.STAT_ACK_CNT (DISPLAY_VALUE),
        subdevice_list[2].STATS.STAT_BACK_CNT (DISPLAY_VALUE)
    }
    VALIDITY  IF ( devices_detected > 0x0002 )   {TRUE;}   ELSE  {FALSE;}
}

MENU subdev_3_stats
{
    LABEL "Subdevice 3";
    ITEMS
    {
//        subdevice_list[3].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        subdevice_list[3].STATS.STAT_STX_CNT (DISPLAY_VALUE),
        subdevice_list[3].STATS.STAT_ACK_CNT (DISPLAY_VALUE),
        subdevice_list[3].STATS.STAT_BACK_CNT (DISPLAY_VALUE)
    }
    VALIDITY  IF ( devices_detected > 0x0003 )   {TRUE;}   ELSE  {FALSE;}
}

MENU subdev_4_stats
{
    LABEL "Subdevice 4";
    ITEMS
    {
//        subdevice_list[4].IDENTITY.LONG_TAG (DISPLAY_VALUE),
        subdevice_list[4].STATS.STAT_STX_CNT (DISPLAY_VALUE),
        subdevice_list[4].STATS.STAT_ACK_CNT (DISPLAY_VALUE),
        subdevice_list[4].STATS.STAT_BACK_CNT (DISPLAY_VALUE)
    }
    VALIDITY  IF ( devices_detected > 0x0004 )   {TRUE;}   ELSE  {FALSE;}
}
#endif

//#if 0 
METHOD forceJoinMode
{
    LABEL "Force Join";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        int iJoin;

        do
        {
            //GET_DEV_VAR_VALUE("Enter Join Mode ", join_mode);
            iassign(join_mode, 1);
            /*#if __TOKVER__ >= 800
                iassign(join_shed_time, 115200000); // search for 1 hour
            #else
                iassign(join_shed_time_int, 115200000); // search for 1 hour
            #endif*/


#if __TOKVER__ >= 800
            GET_DEV_VAR_VALUE("Enter Join Shed Time", join_shed_time);
            GET_DEV_VAR_VALUE("Enter Join Retry Count", join_retries);
#else
            GET_DEV_VAR_VALUE("Enter Join Shed Time", join_shed_time_int);
#endif
            IGNORE_ALL_RESPONSE_CODES();
            IGNORE_ALL_DEVICE_STATUS();
            XMTR_IGNORE_ALL_DEVICE_STATUS();
//jad, according to spec, this command is only for gateways
            send_command(960);
            GET_DEV_VAR_VALUE("Enter Network ID", network_id);
            GET_DEV_VAR_VALUE("Enter Join Key 1", join_key_1);
            GET_DEV_VAR_VALUE("Enter Join Key 2", join_key_2);
            GET_DEV_VAR_VALUE("Enter Join Key 3", join_key_3);
            GET_DEV_VAR_VALUE("Enter Join Key 4", join_key_4);

            DELAY (5, "Sending Join configration in progress");
            send(771,status);
    
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(771, status[STATUS_RESPONSE_CODE]);
            }
    
        }while(status[STATUS_RESPONSE_CODE]);
        send(773,status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(773, status[STATUS_RESPONSE_CODE]);
    }
        send(768,status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(768, status[STATUS_RESPONSE_CODE]);
        }  
        //send_command(769);
    }
 
 
 }
//#endif 
METHOD forceDisconnectMode
{
    LABEL "Force Disconnect";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];

        do
        {
            GET_DEV_VAR_VALUE("Disconnect Network from", network_id);
/*#if __TOKVER__ >= 800
            GET_DEV_VAR_VALUE("Enter Join Shed Time", join_shed_time);
#else
            GET_DEV_VAR_VALUE("Enter Join Shed Time", join_shed_time_int);
#endif*/
//jad, acording to spec, command 960 is only for gateways...
            send(960,status);    
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(960, status[STATUS_RESPONSE_CODE]);
            }
    
        }while(status[STATUS_RESPONSE_CODE]);
        send_command(769);
    }
}

METHOD join_network
{
    LABEL "Join network";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        int iJoin;
 
        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        GET_DEV_VAR_VALUE("Enter Network ID", network_id);

        GET_DEV_VAR_VALUE("Enter Join Key 1", join_key_1);
        GET_DEV_VAR_VALUE("Enter Join Key 2", join_key_2);
        GET_DEV_VAR_VALUE("Enter Join Key 3", join_key_3);
        GET_DEV_VAR_VALUE("Enter Join Key 4", join_key_4);

        send(773,status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(773, status[STATUS_RESPONSE_CODE]);
        }

        send(768,status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(768, status[STATUS_RESPONSE_CODE]);
        }
        
        DELAY (5, "Sending Join configration in progress");
    
/*
       iassign(join_mode, 1);     // join now

#if __TOKVER__ >= 800
       iassign(join_shed_time, 115200000); // search for 1 hour
#else
       iassign(join_shed_time_int, 115200000); // search for 1 hour
#endif

       send(771,status);
       if (status[STATUS_RESPONSE_CODE])
       {
           display_response_status(771, status[STATUS_RESPONSE_CODE]);
       }


        if (ivar_value(join_mode) == 1 ) // only loop if we told device to join now
        {
            do
            {
                send_command(769);
                iJoin = ivar_value(join_status);
                if (iJoin&0x400)
                {
                    DELAY (5, "Join Status - Join complete");
                }
                else if (iJoin&0x040)
                {
                    DELAY (5, "Join Status - Join failed");
                }
                else if (iJoin&0x020)
                {
                    DELAY (5, "Join Status - Join retrying");  
                }
                else if (iJoin&0x200)
                {
                    DELAY (5, "Join Status - Network bandwidth requested");  
                }
                else if (iJoin&0x100)
                {
                    DELAY (5, "Join Status - Network joined");  
                }
                else if (iJoin&0x080)
                {
                    DELAY (5, "Join Status - Network security clearance granted");  
                }
                else if (iJoin&0x010)
                {
                    DELAY (5, "Join Status - Network admission requested");  
                }
                else if (iJoin&0x008)
                {
                    DELAY (5, "Join Status - WirelessHart signal identified");  
                }
                else if (iJoin&0x004)
                {
                    DELAY (5, "Join Status - Wireless time synchronized");  
                }
                else if (iJoin&0x002)
                {
                    DELAY (5, "Join Status - Wireless signal identified");  
                }
                else if (iJoin&0x001)
                {
                    DELAY (5, "Join Status - Network packets heard");  
                }
                else if (iJoin==0)
                {
                    DELAY (5, "Join Status - Network not found");  
                }
                else
                {
                    DELAY (5, "Join Status - Unknown");
                    iJoin = 0;
                } 
                
            } while (!((iJoin & 0x040)||(iJoin & 0x400)));
        }*/
    }
}

//#endif  

METHOD change_burst1_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the burst message is transmitted for";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1;

        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        
#if __TOKVER__ >= 800
//jad
//         GET_DEV_VAR_VALUE("Select sub-device to use for burst message 1", burst_messages[0].SUB_DEVICE_MAPPING);
        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 1", burst_messages2[0].SUB_DEVICE_MAPPING);
        iassign(burst_message_number, 0);
#else
        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 1", burst_message_collection.SUB_DEVICE_MAPPING);
#endif
        send_trans(102, 0, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}

#if __TOKVER__ >= 800
METHOD change_burst2_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the burst message is transmitted for";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1;

        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
//jad
//        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 2", burst_messages[1].SUB_DEVICE_MAPPING);
        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 2", burst_messages2[1].SUB_DEVICE_MAPPING);

        iassign(burst_message_number, 1);
        send_trans(102, 0, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}

METHOD change_burst3_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the burst message is transmitted for";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1;
        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        iassign(burst_message_number, 2);
//jad
//        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 3", burst_messages[2].SUB_DEVICE_MAPPING);
        GET_DEV_VAR_VALUE("Select sub-device to use for burst message 3", burst_messages2[2].SUB_DEVICE_MAPPING);
        send_trans(102, 0, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}

METHOD change_burst4_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the burst message is transmitted for";   
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1; 
        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        iassign(burst_message_number, 3);
//jad
//         GET_DEV_VAR_VALUE("Select sub-device to use for burst message 4", burst_messages[3].SUB_DEVICE_MAPPING);
         GET_DEV_VAR_VALUE("Select sub-device to use for burst message 4", burst_messages2[3].SUB_DEVICE_MAPPING);

        send_trans(102, 0, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}
#endif

#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
METHOD change_event1_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the event is transmitted for";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1;

        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        GET_DEV_VAR_VALUE("Select sub-device to use for Event 1", events[0].EVENT_CONTROL.SUB_DEVICE_MAPPING);

        iassign(eventNumber, 0);
        send_trans(102, 1, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}

METHOD change_event2_subdevice
{
    LABEL "Change sub-device";
    HELP "Changes the sub-device that the event is transmitted for";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        int slen1;

        slen1 = 140;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        GET_DEV_VAR_VALUE("Select sub-device to use for Event 2", events[1].EVENT_CONTROL.SUB_DEVICE_MAPPING);

        iassign(eventNumber, 1);
        send_trans(102, 1, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(102, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}
#endif

METHOD refreshSubDeviceMapping
{
    LABEL "Refresh Sub Device Mapping";
    HELP  "This method forces the sub-device mapping information to be re-read from the field device for all supported burst and event messages.";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        int i;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();


//jad
//ACKNOWLEDGE("total_number_burst_messages %[d]{total_number_burst_messages}");
//Test device returned 10 for total_number_burst_messages even though the array only has 4 elements
// so limited the loop to 4 iterations
        for(i=0; i<ivar_value(total_number_burst_messages); i++)
        {
//fbi	#if __TOKVER__ >= 800)
//fbi	            iassign(burst_message_number, i);
//fbi	#endif

//jad
            if (i>3)
                break;
//jad end
            send_trans(101, 0, status);
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(101, status[STATUS_RESPONSE_CODE]);
            }
        }

#if (__TOKVER__ >= 600) && (INCLUDE_EVENTS)
        for(i=0; i<ivar_value(number_events_supported); i++)
        {
            iassign(eventNumber, i);
            send_trans(101, 1, status);
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(101, status[STATUS_RESPONSE_CODE]);
            }
        }
#endif
        ACKNOWLEDGE("Subdevice mapping has been refreshed for all Burst and Event messages.");
        process_abort();
    }
}



MENU power_setup
{
    LABEL "Power setup";
    ITEMS
    {
       Power,
       power_setup_option(DISPLAY_VALUE,READ_ONLY)
    }

}
 
COMMAND Power_Setup_command
{
    NUMBER 128;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            power_setup_option        
        }
        REPLY
        {
            response_code, device_status,
            power_setup_option
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND Read_Power_Setup
{
    NUMBER 129;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, power_setup_option
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error]; 
        32, MODE_ERROR, [busy];
    }
}

METHOD Power
{
    LABEL "Select power option";
    HELP  "This method allows to choose the power setup option . eg. 1 = 24V Power: Source Enable , 2 = 24V Power: Source Disable , 3 = Loop Power ";
    CLASS DEVICE;
    DEFINITION 
    {               
       GET_DEV_VAR_VALUE("Enter new power option ",power_setup_option);
       send_command(128);
//jad
//       acknowledge("New power option is selected",power_setup_option);
       ACKNOWLEDGE("New power option is selected %{power_setup_option}");


       process_abort();
    }
}

METHOD Read_Power_Status
{
    LABEL "Read current power option";
    HELP  "This method reads the current power setup option ";
    CLASS DEVICE;
    DEFINITION 
    {   
        send_command(129); 
        ACKNOWLEDGE("Power option is updated with current setting %{power_setup_option}");
        process_abort();

    }
}



METHOD ProcessCommandMethod
{
    LABEL "Confirm Process Command";
    HELP  "";
    CLASS DEVICE;
    DEFINITION 
    {
        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        iassign(burst_message_number, 0);  
//jad
//        burst_messages[0].COMMAND_NUMBER = ProcessCommand; 
        burst_messages2[0].COMMAND_NUMBER = ProcessCommand;  
        burst_command_number = ProcessCommand;
        send_command(108);
        ACKNOWLEDGE("Process command is updated with new settings");

    }
}

METHOD StatusCommandMethod 
{
    LABEL "Confirm Status Command";
    HELP  "";
    CLASS DEVICE;
    DEFINITION 
    {
        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        GET_DEV_VAR_VALUE("Enter Status Command ",StatusCommand);
        iassign(burst_message_number, 1);
//jad
//        burst_messages[1].COMMAND_NUMBER = StatusCommand;      
        burst_messages2[1].COMMAND_NUMBER = StatusCommand;      
        send_command(108); 

    }
}

METHOD DiscoverSubDevices
{
    
    LABEL "Discover Sub Devices";
    HELP  "This Method will send a custom command for sub device re-discovery";
    
    DEFINITION 
    {
        char status[STATUS_SIZE];

        int i;
        int time;


        //jad
//        DELAY (5, "Sub Device Rediscovery in progress, DD will close now. Please restart DD after 2 minutes");
        DELAY (5, "Sub Device Rediscovery in progress. Please restart Host after 2 minutes");
        send_trans(132, 0, status);
//        while((status[STATUS_RESPONSE_CODE])== 0)
//        {
//            process_abort();
 //       }

//jad
//Count down timer here for 2 minutes
// 
        for (i=0;i<24;i++)
        {
            time = i*5;
            DELAY(5,"Waiting for 2 minutes..."
                    "\n\nElapsed Time: %[d]{time} seconds");

        }
        ACKNOWLEDGE("Please restart Host");

    }
}

COMMAND command132
{
    NUMBER 132;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

MENU Joining_offline
{
    LABEL "Joining";
    ITEMS
    {
        Join_Status(DISPLAY_VALUE)
                
    }
}
MENU Joining
{
    LABEL "Joining";
    ITEMS
    {
        Join_Status(DISPLAY_VALUE),
        forceJoinMode,
        forceDisconnectMode
                
    }
}

/*
VARIABLE FastJoin
{
    LABEL "HART Join";
    HELP  "";
    CLASS DYNAMIC;
    HANDLING READ;
    //#if __TOKVER__ >= 602
    DEFAULT_VALUE 1;
   // #endif
    TYPE ENUMERATED
    {
        { 1, "OFF", "help" },
        { 2, "ON", "help"}
    }
} */
//jad
//This is a "disconnect" command
COMMAND command960
{
    NUMBER 960;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
//jad
//            0
            user_requested

        }
        REPLY
        {

//jad
//firmware has 2 bytes returning that look like garbage
//fixed in 144? put back to normal
//            response_code, device_status
//nope, still behaves incorrectly
            response_code, device_status, jad_junk
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
//jad for above command

VARIABLE user_requested
{
    LABEL "junk";
    HELP  "";
    CLASS LOCAL;
    TYPE INTEGER (1);
    DEFAULT_VALUE 0;
}

VARIABLE jad_junk
{
    LABEL "junk";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ;
    TYPE INTEGER (2);
}





//JAD
//This is in wireless, why not imported?

COMMAND command771
{
    NUMBER 771;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            join_mode, join_shed_time, join_retries
        }
        REPLY
        {
            response_code, device_status, join_mode, join_shed_time, join_retries
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}




//jad, common status definition has this as 2 bytes, but only defined as 1 here
VARIABLE Join_Status
{
    LABEL "Join Status";
    HELP  "";
    CLASS DYNAMIC;
    HANDLING READ;
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 2;
    #endif
    TYPE ENUMERATED 
    {
        { 0, "Init", "The mote is in the process of booting." },
        { 1, "Idle", "help" },
        { 2, "Searching", "help" },
        { 3, "Negotiating", "help" },
        { 4, "Connected", "help" },
        { 5, "Operational", "help" },
        { 6, "Disconnected", "help" }
    }
   
}

VARIABLE join_shed_time
{
    LABEL "Join Shed Time";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    TYPE TIME_VALUE;
}

VARIABLE join_retries
{
    LABEL "Join Retry Count";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    DEFAULT_VALUE 5;
    #endif
    TYPE INTEGER;
}

METHOD Join_Status_WHA
{
    
    LABEL "Join_Status";
    HELP  "";
    
    DEFINITION 
    {
            IGNORE_ALL_RESPONSE_CODES();
            IGNORE_ALL_DEVICE_STATUS();
            XMTR_IGNORE_ALL_DEVICE_STATUS();
            send_command(769);
    }
}
/*
METHOD HARTFastJoin
{
    LABEL "HART Join";
    HELP  "";
    DEFINITION 
    {
        GET_DEV_VAR_VALUE("Enter HART Join ", FastJoin);
        send_command(134);
    }
}

COMMAND HART_JOIN
{
    NUMBER 134;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            FastJoin
        }
        REPLY
        {
            response_code, device_status,FastJoin
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND HART_JOIN_READ
{
    NUMBER 135;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {}
        REPLY
        {
            response_code, device_status,FastJoin
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}*/
//Join_Status

// mhu - added dummy variable with 2 bytes length
VARIABLE constZero2
{
    /* LABEL and HELP not defined - this is never displayed to a user! */
    CLASS LOCAL;
    TYPE UNSIGNED_INTEGER(2)
    {
        MIN_VALUE 0x00;  //This is to help for hosts which do not implement DEFAULT_VALUE properly
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 0x00;
#endif
}
